# -*- coding: utf-8 -*-
"""
Created on Fri Mar 23 16:29:00 2018

@author: Callie Darsey

Calculate optical flow forwards and backwards for a clip.
"""

import cv2
import pickle
import numpy as np


# video path
common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific_path = 'STI Polk Apr06 C/'
out_folder = 'MVI_0029_trans/'
video_name = 'MVI_0029_trans.mp4'

video_path = common_path + specific_path + video_name

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = 4

'''Resolution rescaling factor'''
scaleFactor = 0.67


'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''
pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0

      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2. CAP_PROP_FPS )

print('number of frames:', num_frames)
print('frames per second:', fps)

print('framesToAdvance:', framesToAdvance)
print('scaleFactor:', scaleFactor)


# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

# collect info about the flow magnitude in the top quarter
mins = []
medians = []
means = []
maxs = []


# loop through the frames

e1 = cv2.getTickCount()

count = 0


while(cap.isOpened()):
    
    eb = cv2.getTickCount()
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            break
     
    if success_flag:
    # process upon true read frame success_flag

        #initialize frame2
        frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
        frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
        
        eif = cv2.getTickCount()

        # calculate the flow between frame1 and frame2    
        flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                            flow=None,
                                            pyr_scale=pyr_scale,
                                            levels=levels,
                                            winsize=winsize,
                                            iterations=iterations,
                                            poly_n=poly_n,
                                            poly_sigma=poly_sigma,
                                            flags=flags)


        flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
        flow_mag_sorted = np.sort(flow_mag, axis=None)
        highest_value_quarter = \
            flow_mag_sorted[-int(len(flow_mag_sorted) / 4):]
        
        mins.append(np.min(highest_value_quarter))
        medians.append(np.median(highest_value_quarter))
        means.append(np.mean(highest_value_quarter))
        maxs.append(np.max(highest_value_quarter))
        
        eef = cv2.getTickCount()
        print('{:04d}'.format(count - framesToAdvance),
              'flow calc {:.2f} seconds'.format(\
                         (eef - eif)/ cv2.getTickFrequency())) 

        
        # pickle result
        pickle_path = common_path + specific_path  + out_folder + 'flow_' + \
            video_name[0:-4] + \
            '_{:04d}_{:04d}'.format(count - framesToAdvance, count) +  '.data'
            
        with open(pickle_path, 'wb') as f:
            pickle.dump((frame1, frame2, flow), f)
            
        
        # update frame1
        frame1 = frame2
        frame1_gray = frame2_gray
        
        
        
        ee = cv2.getTickCount()
        print('{:04d}'.format(count - framesToAdvance),
              'flow calc {:.2f} seconds'.format(\
                         (ee - eb) / cv2.getTickFrequency()))

    else:

        # release capture
        cap.release()
        
        with open(video_path[0:-3:] + 'flow_mag_info.data', 'wb') as f:
                    pickle.dump((mins, medians, means, maxs), f)


e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)
