# -*- coding: utf-8 -*-
"""
Created on Sun Mar 25 11:59:08 2018

@author: Callie Darsey

Play with arrows and marking the largest magnitude flow
"""

import cv2
import numpy as np
import pickle
import os
import fns_investigating_optical_flow_01 as fns

# video path
common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific_path = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
video_name = 'MVI_0028_trans__200_208.mp4'


directory = common_path + specific_path

dir_list = os.listdir(directory)

collect_listings = []

for listing in dir_list:
    
    if 'flow_' + video_name[0:-4] in listing and listing[-5:] == '.data':
        
        collect_listings.append(listing)

collect_listings.sort()

# prep window
title = 'display window'
move_window = [60, -30]
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, move_window[0], move_window[1])

for listing in collect_listings:
    
    # pickle result
    pickle_path = common_path + specific_path + listing
        
    with open(pickle_path, 'rb') as f:
        frame1, frame2, flow = pickle.load(f)
        
    flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
    
    max_flow_mag = np.max(flow_mag)
    
    col_ind, row_ind = np.where(flow_mag == max_flow_mag)
    
    arrows = np.copy(frame1)    
    arrows = fns.draw_flow(arrows, flow,
                           np.ones(frame1.shape[0:2], dtype=np.uint8),
                           step=4)
    
    
    # draw the text
    cv2.putText(arrows, str(int(listing[-14:-10])),
                (arrows.shape[1] - 100, 25),
                cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)    
    
    
    center = (row_ind[0], col_ind[0])
    radius = 10
    color = (0, 0, 255)
    thickness = 3
    
    cv2.circle(arrows, center, radius, color, thickness)
    
    # draw the text
    cv2.putText(arrows, '{:.2f}'.format(max_flow_mag),
                (row_ind[0] + 15, col_ind[0] ),
                cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 255), 2)  

    with open(pickle_path, 'wb') as f:
         pickle.dump((frame1, frame2, flow, arrows), f)      

    cv2.imshow(title, arrows)
    
    cv2.waitKey(1)
    
cv2.destroyAllWindows()

