# -*- coding: utf-8 -*-
"""
Created on Sat Mar 31 13:12:27 2018

@author: Callie Darsey


"""

# -*- coding: utf-8 -*-
"""
Created on Wed Mar 28 12:11:21 2018

@author: Callie Darsey


Arg. Look into trigger and predict flow
"""


import cv2
import numpy as np
import pickle
import os
import fns_investigating_optical_flow_01 as fns
from matplotlib import pyplot as plt
import get_lessened_features_and_save_steps_01 as get
import overlay_01 as overlay
import component_functions_01 as comp_fns


def form_regions_via_keep_and_trigger(frame, flow, cc_trigger,
                                      way_1_component_mask,
                                      way_2_component_mask,
                                      way_3_component_mask,
                                      way_1_model,
                                      way_2_model,
                                      way_3_model, path, fnum):
    '''
    Return the regions mask found from trigger mask overlap with
    regions classified as keep in all three ways for forming components
    for classification. If there is no overlap, return None.
    
    These image are also saved along the way:
        bgr with way 1 components class prediction overlaid
        bgr with way 2 components class prediction overlaid
        bgr with way 3 components class prediction overlaid
        display keep components summed and trigger
        bgr with region overlay if regions formed

    '''
    
    # get the connected component structures for each way_#_mask
    connectivity = 4
    cc_way_1 = cv2.connectedComponents(way_1_component_mask,
                                       connectivity, cv2.CV_32S)

    cc_way_2 = cv2.connectedComponents(way_2_component_mask,
                                       connectivity, cv2.CV_32S)

    cc_way_3 = cv2.connectedComponents(way_3_component_mask,
                                       connectivity, cv2.CV_32S)
    
    # get the features in the added to the cv2.connectedComponents() structures
    cc_way_1 = get.get_features(cc_way_1, frame, flow)
    cc_way_2 = get.get_features(cc_way_2, frame, flow)
    cc_way_3 = get.get_features(cc_way_3, frame, flow)
    
    # get the classification prediction in cv2.connectedComponents() structures
    cc_way_1 = get.classify(cc_way_1, way_1_model)
    cc_way_2 = get.classify(cc_way_2, way_2_model)
    cc_way_3 = get.classify(cc_way_3, way_3_model)
    
    '''
    save bgr classification overlay prediction image
    and collect the keep masks from the three ways
    '''
    collect_keep_masks = []
    for tag, cc in zip(['way_1_', 'way_2_', 'way_3_'],
                       [cc_way_1, cc_way_2, cc_way_3]):
        
        toss_ind = np.where(cc[-1] == 0)[0]
        
        bgr_comp_overlay = overlay.overlay(frame, cc,
                                           np.unique(cc[1])[toss_ind],
                                           (0, 0, 255), alpha=0.5)
        
        keep_ind = np.where(cc[-1] == 1)[0]
        
        bgr_comp_overlay, keep_mask = get.overlay(bgr_comp_overlay, cc,
                                                  np.unique(cc[1])[keep_ind],
                                                  (0, 255, 0), alpha=0.5)
        
        cv2.imwrite(path + tag + 'overlay_' + fnum + '.png', bgr_comp_overlay)
                
        collect_keep_masks.append(keep_mask)
        
    # sum up the keep component masks
    keep_sum = np.sum(collect_keep_masks, axis=0)
    
    # save sum of keep components and the trigger mask components image
    keep_sum_display = ((keep_sum + 1) ** 4 - 1).astype(np.uint8)
    
    keep_3_and_trigger = overlay.overlay(keep_sum_display, cc_trigger,
                                         np.unique(cc_trigger[1])[1:],
                                         (255, 0, 255), alpha=0.5)
    
    cv2.imwrite(path + 'sum_of_keep_comp_and_trigger_comp_' + fnum + '.png',
                keep_3_and_trigger)
    
    '''
    find where the trigger components overlap the where the keep components
    sum is three
    '''    
    cc_keep_sum_three = \
        cv2.connectedComponents((keep_sum[:, :, 0] == 3).astype(np.uint8),
                                connectivity, cv2.CV_32S)
    
    collect_comp = [np.zeros(frame.shape[0:2])]
    
    for label_3 in np.unique(cc_keep_sum_three[1])[1:]:
        
        keep_sum_3_comp = (cc_keep_sum_three[1] == label_3)
        
        for label_trigger in np.unique(cc_trigger[1])[1:]:

            trigger_comp = (cc_trigger[1] == label_trigger)
            
            nzcount = np.count_nonzero(keep_sum_3_comp * trigger_comp)
          
            if nzcount > 0:
                                
                collect_comp.append(keep_sum_3_comp)
                collect_comp.append(trigger_comp)
    
    # form the region mask
    regions_mask = np.sum(collect_comp, axis=0)
    
    # if regions were found, fill holes and return
    if np.count_nonzero(regions_mask) > 0:

        return comp_fns.fill_holes(regions_mask.astype(np.uint8))
      
    # otherwise return None
    else:
        
        return None
    

def adjust_forward(mask, flow):
    '''
    Adjust the mask forward according to the flow and return.
    '''
    
    row, col = np.where(mask)
        
    flow_row = flow[:, :, 0][row, col]
    flow_col = flow[:, :, 1][row, col]
    
    row_adjust = (row + flow_row).astype(np.int)
    col_adjust = (col + flow_col).astype(np.int)
    
    good_ind = np.where(np.all([row_adjust >= 0,
                                row_adjust < mask.shape[0],
                                col_adjust >= 0,
                                col_adjust < mask.shape[1]], axis=0))[0]
    
    if len(good_ind) < len(row_adjust):
        
        row_adjust = row_adjust[good_ind]
        col_adjust = col_adjust[good_ind]
    
    adjusted_forward_mask = np.zeros_like(mask)
    
    adjusted_forward_mask[row_adjust, col_adjust] = 1
    
    # close the adjusted forward region to close holes
    close_adjusted_forward_mask = \
        cv2.morphologyEx(adjusted_forward_mask.astype(np.uint8),
                         cv2.MORPH_CLOSE,np.ones((3,3)))
    
    return close_adjusted_forward_mask
        
  
def get_adjusted_forward_labels(flow, prior_regions_mask, regions_mask):
    '''
    This function adjusts forward the components in the regions_label_matrix.
    If the adjusted forward component overlaps components of the 
    new_regions_label_matrix, then those labels are associated.
    
    Returns a dictionary {region label i: associated new region labels}
    '''   
    # dictionary to return
    d = {}
    
    # get the label matrices
    connectivity = 4
    
    cc_prior_regions = cv2.connectedComponents(prior_regions_mask,
                                               connectivity, cv2.CV_32S)
    
    cc_regions = cv2.connectedComponents(regions_mask, connectivity,
                                         cv2.CV_32S)
    
    # loop through all the regions
    for rlabel in np.unique(cc_prior_regions[1])[1:]:
        
        # get the mask for the one region
        region_mask = (cc_prior_regions[1] == rlabel)
    
        # adjust the one region forward
        region_adjusted_forward_mask = adjust_forward(region_mask, flow)
        
        # find the labels in the all regions adjusted forward mask
        adjusted_foward_labels = np.unique(region_adjusted_forward_mask * \
                                           cc_regions[1])[1:]
        
        d[rlabel] = adjusted_foward_labels
    
    # return the dictionary
    return d

def add_labels(dict_have, dict_add):
    '''
    Return dictionary of form {region label i : [hist for label i region,
                                                 associated new region labels]}
    
    Inputs:
        dict_have - {region label i : [hist for label i region]}
        dict_add - {region label i: associated new region labels}
    '''
    
    for key in dict_have.keys():
        
        dict_have[key].append(dict_add[key])
        
    return dict_have

def get_cc_adjusted_forward(regions_mask, flow):
    '''
    Adjust forward a regions mask and return the cv2.connectedComponents()
    output which holds the adjusted regions label matrix in its second
    component.
    '''
    
    # adjust the hole map and get the label matrix
    adjusted_forward_mask = adjust_forward(regions_mask, flow)
        
    connectivity = 4
    cc_adjusted_forward = cv2.connectedComponents(adjusted_forward_mask,
                                                  connectivity, cv2.CV_32S)
    
    return cc_adjusted_forward
    
    

def get_dict_with_hist(regions_mask, flow):
    '''
    This function returns the information dictionary
    
    dictionary entry holds {region label i : [hist for label i region]}
    
    '''

    # get the label matrix of regions_mask
    cc_regions = cv2.connectedComponents(regions_mask,
                                         connectivity, cv2.CV_32S)
    
    # form the dictionary
    d = {}
    
    # loop through all the regions
    for rlabel in np.unique(cc_regions[1])[1:]:
        
        # get the mask for the one region
        region_mask = (cc_regions[1] == rlabel)
        
        # find the hist in that region
        bin_sums, \
        bin_sums_div_by_count, \
        bin_sums_div_by_area = \
            get.get_flow_mag_in_phase_bins(flow, region_mask,
                                           -np.pi / 8, np.pi / 4)
        
        # add dictionary entry
        d[rlabel] = [bin_sums_div_by_count]
        
    return d
        
        
    
def form_regions(cc_adjusted_forward, cc_trigger):
    '''
    This function returns two masks (or None):
        - mask for combined regions where adjusted forward component and
          trigger component overlapped
        - mask for trigger regions that did not match adjusted forward regions
    '''
    
    # collect components masks for trigger regions and adjusted forward regions
    # that overlapped
    collect_joined = [np.zeros(cc_trigger[1].shape)]
    
    # collect components mask for trigger regions that did not overlap any
    # adjusted forward regions
    collect_remaining = [np.zeros(cc_trigger[1].shape)]
    
    # loop through the trigger regions
    for trigger_label in np.unique(cc_trigger[1])[1:]:
        
        trigger_label_mask = (cc_trigger[1] == trigger_label)
        
        # save size of collected_joined
        len_collected_joined = len(collect_joined)
        
        # loop through the adjusted forward components
        for adjusted_label in np.unique(cc_adjusted_forward[1])[1:]:
            
            adjusted_label_mask = (cc_adjusted_forward[1] == adjusted_label)

            # the adjusted forward component is always added
            collect_joined.append(adjusted_label_mask)
            
            # increase size by one
            len_collected_joined = len_collected_joined + 1
            
            # trigger component added if it overlapped adjusted forward mask
            if np.count_nonzero(trigger_label_mask * adjusted_label_mask) > 0:
                
                collect_joined.append(trigger_label_mask)
                
        
        '''
        if collect_joined size is not greater than just having adjusted
        forward components added, the trigger region remains
        '''
        if len_collected_joined == len(collect_joined):
            
            collect_remaining.append(trigger_label_mask)
            
    # form the two masks
    joined_mask = np.any(collect_joined, axis=0)
    
    remaining_mask = np.any(collect_remaining, axis=0)
    
    # if the mask is empty, replace it with None
    if np.count_nonzero(joined_mask) == 0:
        
        joined_mask = None
    
    if np.count_nonzero(remaining_mask) == 0:
        
        remaining_mask = None
        
    # return the masks
    return (joined_mask, remaining_mask)
    

    

def break_up_and_link(region_mask):
    '''
    '''













# video path
common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific_path = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
video_name = 'MVI_0028_trans__200_208.mp4'

# gather the .data paths to saved frame1, frame1, flow for a clip
directory = common_path + specific_path

dir_list = os.listdir(directory)

collect_listings = []

for listing in dir_list:
    
    if 'flow_' + video_name[0:-4] in listing and listing[-5:] == '.data':
        
        collect_listings.append(listing)

collect_listings.sort()


# prep window
title = 'display window'
move_window = [60, -30]
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, move_window[0], move_window[1])


# get the gathered samples for trigger flow magnitude
with open(common_path + specific_path[0:32] + \
          'flow_mag_samples.data', 'rb') as f:
    
    flow_mag_samples, specific_paths, video_names, exemplar_names = \
        pickle.load(f)


bp1 = plt.boxplot(\
    np.reshape(np.sort(flow_mag_samples), -1)[int(len(flow_mag_samples) / 2):])
plt.show()


Q1 = np.min(bp1['boxes'][0].get_data()[1])
Q3 = np.max(bp1['boxes'][0].get_data()[1])
median = bp1['medians'][0].get_data()[1][0]
Q3_plus_1pt5_IQR = np.max(bp1['caps'][1].get_data()[1]) # IQR = Q3 - Q1


# make the review folder
adjust_review_01 = common_path + specific_path + "adjust_review_03/"
if not os.path.isdir(adjust_review_01):
    os.mkdir(adjust_review_01)
    
# region masks
region_masks = adjust_review_01 + "region_masks/"
if not os.path.isdir(region_masks):
    os.mkdir(region_masks)
    

# load the classification models
with open('./rfc_models/from_wheeler/' + \
          'way_1_gs_rf_clf_64y2018m03d26-h15m12s08.211352.data', \
          'rb') as f:
    way_1_model = pickle.load(f)    

with open('./rfc_models/from_wheeler/' + \
          'way_2_gs_rf_clf_64y2018m03d27-h00m19s13.508701.data', \
          'rb') as f:
    way_2_model = pickle.load(f)   
    
with open('./rfc_models/from_wheeler/' + \
          'way_3_gs_rf_clf_64y2018m03d26-h18m14s15.655086.data', \
          'rb') as f:
    way_3_model = pickle.load(f)   

'''flow_MVI_0028_trans__200_208_0000_0004.data'''

# will hold None or dictionary entries per frame visited
# dictionary entry holds {region labeli : [hist for labeli region,
#                                          adjusted forward labels]}
collect = []

for listing in collect_listings:
    
    # frame number
    nframe1 = listing[-14:-10]
    
    # pickle result
    pickle_path = common_path + specific_path + listing
    
    # load pickled flow    
    with open(pickle_path, 'rb') as f:
        frame1, frame2, flow = pickle.load(f)
    
    # threshold the flow magnitude to obtain trigger regions    
    flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
    flow_mag_trigger_mask = np.logical_and(flow_mag > Q1, flow_mag < Q3)
    
    # save and display the trigger regions
    trigger_arrows = np.copy(frame1)
    fns.draw_flow(trigger_arrows, flow, flow_mag_trigger_mask, step=4)
    
    # draw the text
    cv2.putText(trigger_arrows, str(int(nframe1)),
                (trigger_arrows.shape[1] - 100, 25),
                cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)    

    cv2.imshow(title, trigger_arrows)
    
    # escape key
    if cv2.waitKey(1) == 27:
        
        break

    cv2.imwrite(adjust_review_01 + 'flow_trigger_arrows_' + nframe1 + '.png', \
                trigger_arrows)

    # there are trigger regions in the frame
    if np.count_nonzero(flow_mag_trigger_mask) > 0:
        
        # first frame or no region information recorded last frame
        if len(collect) == 0 or collect[-1] is None:
            
            print(nframe1, 'got trigger; no record for previous frame')
            
            # get the components via the three ways
            intermediate_tag, intermediate_arr = \
                get.get_components_and_intermediate_steps(frame1, flow, Q1)
            
            # form the regions mask
            connectivity = 4
            cc_trigger = cv2.connectedComponents(\
                                        flow_mag_trigger_mask.astype(np.uint8),
                                        connectivity, cv2.CV_32S)
            
            region_mask = \
                form_regions_via_keep_and_trigger(frame1, flow, cc_trigger,
                                                  intermediate_arr[6],
                                                  intermediate_arr[13],
                                                  intermediate_arr[15],
                                                  way_1_model,
                                                  way_2_model,
                                                  way_3_model,
                                                  adjust_review_01, nframe1)
                
            # check if we found regions
            if region_mask is None:
                
                collect.append(None)
                
            # find the flow hist in regions
            else:
                
                # save the regions mask
                with open(region_masks + 'region_mask_' + nframe1 + \
                          '.data', 'wb') as f:
                    pickle.dump((region_mask), f)
                                    
                # save image displaying the regions overlay in bgr
                cc_regions = cv2.connectedComponents(region_mask,
                                                     connectivity, cv2.CV_32S)
                
                regions_overlay = overlay.overlay(frame1, cc_regions,
                                                  np.unique(cc_regions[1])[1:],
                                                  (28, 171, 237), alpha=0.5)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_regions_' + \
                            nframe1 + '.png', regions_overlay)
                
                # save image with flow arrows
                arrows = np.copy(frame1)
                fns.draw_flow(arrows, flow, np.ones(frame1.shape[0:2]), step=4)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_arrows_' + \
                            nframe1 + '.png', arrows)
                
                # get the dictionary to save for these regions
                dict_info = get_dict_with_hist(region_mask, flow)
                
                # append
                collect.append(dict_info)
                     
        
        # region information was collected last frame
        else:
            
            print(nframe1, 'got trigger; have record for previous frame')
            
            # adjust forward
            cc_adjusted_forward = get_cc_adjusted_forward(region_mask, flow)

            # save bgr with recorded adjusted forward info
            adjusted_forward_overlay = \
                overlay.overlay(frame1, cc_adjusted_forward,
                                np.unique(cc_adjusted_forward[1])[1:],
                                (255, 255, 0), alpha=0.5)
                
            cv2.imwrite(adjust_review_01 + \
                        'bgr_overlay_recorded_adjusted_forward_' + \
                        nframe1 + '.png', adjusted_forward_overlay)
            
            # make the two maps               
            cc_trigger = \
                cv2.connectedComponents(flow_mag_trigger_mask.astype(np.uint8),
                                        connectivity, cv2.CV_32S)
            
            joined_mask, remaining_trigger_mask = \
                form_regions(cc_adjusted_forward, cc_trigger)
                
            # see if we need to classify way components
            if remaining_trigger_mask is not None:
                
                # get the components via the three ways
                intermediate_tag, intermediate_arr = \
                    get.get_components_and_intermediate_steps(frame1, flow, Q1)

                # form the regions mask
                connectivity = 4
                
                cc_remaining_trigger = cv2.connectedComponents(\
                    remaining_trigger_mask.astype(np.uint8), connectivity,
                    cv2.CV_32S)
                
                trigger_and_way_mask = \
                    form_regions_via_keep_and_trigger(frame1, flow,
                                                      cc_remaining_trigger,
                                                      intermediate_arr[6],
                                                      intermediate_arr[13],
                                                      intermediate_arr[15],
                                                      way_1_model,
                                                      way_2_model,
                                                      way_3_model,
                                                      adjust_review_01,
                                                      nframe1)
                    
                # when a mask is found from trigger and way keep
                if  trigger_and_way_mask is not None:  
                    
                    # save bgr image with the regions from trigger and keep
                    cc_trigger_and_way_regions = \
                        cv2.connectedComponents(trigger_and_way_mask,
                                                connectivity, cv2.CV_32S)
                        
                    trigger_and_way_regions_overlay = \
                        overlay.overlay(frame1, cc_trigger_and_way_regions,
                                np.unique(cc_trigger_and_way_regions[1])[1:],
                                (255, 0, 255), alpha=0.5)
                        
                    cv2.imwrite(adjust_review_01 + \
                            'bgr_overlay_trigger_and_way_regions_' + \
                            nframe1 + '.png', trigger_and_way_regions_overlay)                
                
            else:
                
                trigger_and_way_mask = None
            
            '''combine the maps into region mask'''
            
            # both masks got regions
            if joined_mask is not None and trigger_and_way_mask is not None:
                
                mask = np.any([joined_mask, trigger_and_way_mask], axis=0)
                
                # rename the prior region mask
                region_mask_prior = region_mask                
                
                # fill the holes and set as region mask
                region_mask = comp_fns.fill_holes(mask.astype(np.uint8))
                
            # only got regions from adjusted side    
            elif joined_mask is not None and trigger_and_way_mask is None:
                
                mask = joined_mask
                
                # rename the prior region mask
                region_mask_prior = region_mask
                
                # fill the holes and set as region mask
                region_mask = comp_fns.fill_holes(mask.astype(np.uint8))
                
            # only got regions from the trigger and way side
            elif joined_mask is None and trigger_and_way_mask is not None:
                
                mask = trigger_and_way_mask
                
                # rename the prior region mask
                region_mask_prior = region_mask
                
                # fill the holes and set as region mask
                region_mask = comp_fns.fill_holes(mask.astype(np.uint8))
                
            # neigther map got anything
            elif joined_mask is None and trigger_and_way_mask is None:
                
                # rename the prior region mask
                region_mask_prior = region_mask                
                
                region_mask = None
                
            
            '''Append associated labels to last entry, and add new entry'''
            
            # verify region(s) were found for current frame
            if region_mask is None:
                
                collect.append(None)
            
            else:
                
                # associate labels and save
                dict_adjusted_forward = \
                    get_adjusted_forward_labels(flow, region_mask_prior,
                                                region_mask)
                    
                collect[-1] = add_labels(collect[-1], dict_adjusted_forward)
                
                
                # save the regions mask
                with open(region_masks + 'region_mask_' + nframe1 + \
                          '.data', 'wb') as f:
                    pickle.dump((region_mask), f)
                                    
                # save image displaying the regions overlay in bgr
                cc_regions = cv2.connectedComponents(region_mask,
                                                     connectivity, cv2.CV_32S)
                
                regions_overlay = overlay.overlay(frame1, cc_regions,
                                                  np.unique(cc_regions[1])[1:],
                                                  (0, 255, 255), alpha=0.5)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_regions_' + \
                            nframe1 + '.png', regions_overlay)
                
                # save image with flow arrows
                arrows = np.copy(frame1)
                fns.draw_flow(arrows, flow, np.ones(frame1.shape[0:2]), step=4)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_arrows_' + \
                            nframe1 + '.png', arrows)                
                
                # get the dictionary to save for these regions
                dict_info = get_dict_with_hist(region_mask, flow)
                
                # append
                collect.append(dict_info)
       
        
    # there are not trigger regions in the frame
    else:
        
        # first frame or no region information recorded last frame
        if len(collect) == 0 or collect[-1] is None:
            
            print(nframe1, 'no trigger; no record for previous frame')            
            
            collect.append(None)
        
        # region information was collected last frame
        else:
            
            print(nframe1, 'no trigger; have record for previous frame')
            
            # adjust forward
            cc_adjusted_forward = get_cc_adjusted_forward(region_mask, flow)

            # save bgr with recorded adjusted forward info
            adjusted_forward_overlay = \
                overlay.overlay(frame1, cc_adjusted_forward,
                                np.unique(cc_adjusted_forward[1])[1:],
                                (255, 255, 0), alpha=0.5)
                
            cv2.imwrite(adjust_review_01 + \
                        'bgr_overlay_recorded_adjusted_forward_' + \
                        nframe1 + '.png', adjusted_forward_overlay) 

            '''form region mask'''

            # adjusted had regions
            if np.count_nonzero(cc_adjusted_forward[1]) > 0:
                
                mask = cc_adjusted_forward[1]
                
                # rename the prior region mask
                region_mask_prior = region_mask                
                
                # fill the holes and set as region mask
                region_mask = comp_fns.fill_holes(mask.astype(np.uint8))
                
            # adjusted forward did not have regions
            else:
                               
                # rename the prior region mask
                region_mask_prior = region_mask                
                
                region_mask = None


            '''Append associated labels to last entry, and add new entry'''
            
            # verify region(s) were found for current frame
            if region_mask is None:
                
                collect.append(None)
            
            else:
                
                # associate labels and save
                dict_adjusted_forward = \
                    get_adjusted_forward_labels(flow, region_mask_prior,
                                                region_mask)
                    
                collect[-1] = add_labels(collect[-1], dict_adjusted_forward)
                
                
                # save the regions mask
                with open(region_masks + 'region_mask_' + nframe1 + \
                          '.data', 'wb') as f:
                    pickle.dump((region_mask), f)
                                    
                # save image displaying the regions overlay in bgr
                cc_regions = cv2.connectedComponents(region_mask,
                                                     connectivity, cv2.CV_32S)
                
                regions_overlay = overlay.overlay(frame1, cc_regions,
                                                  np.unique(cc_regions[1])[1:],
                                                  (0, 255, 255), alpha=0.5)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_regions_' + \
                            nframe1 + '.png', regions_overlay)
                
                # save image with flow arrows
                arrows = np.copy(frame1)
                fns.draw_flow(arrows, flow, np.ones(frame1.shape[0:2]), step=4)
                
                cv2.imwrite(adjust_review_01 + 'bgr_overlay_arrows_' + \
                            nframe1 + '.png', arrows)                
                
                # get the dictionary to save for these regions
                dict_info = get_dict_with_hist(region_mask, flow)
                
                # append
                collect.append(dict_info)

            
cv2.destroyAllWindows()
