# -*- coding: utf-8 -*-
"""
Created on Sat Feb 17 17:11:37 2018

@author: Callie Darsey

component_functions_01
"""

import pickle
import numpy as np
import cv2
import skin_detector_01 as skin_detector
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import overlay_01 as overlay


def fill_holes(mask):
    '''
    Fill the holes in connected components in the mask.
    
    References:
        https://www.learnopencv.com/filling-holes-in-an-image-using-opencv-python-c/
        
        http://answers.opencv.org/question/9863/fill-holes-of-a-binary-image/
    '''

    assert mask.dtype == np.uint8, 'fill_holes: mask expected to be UINT8'
    
    # force nonzero values to be 255
    mask[mask != 0] = 255
    
    # make a zero border around image    
    mask_floodfill = np.zeros((mask.shape[0] + 2, mask.shape[1] + 2),
                              dtype=np.uint8)
    mask_floodfill[1:mask.shape[0] + 1, 1:mask.shape[1] + 1] = mask
    
    # starting from the zero border, do a flood fill
    cv2.floodFill(mask_floodfill, None, (0, 0), 255)
    
    # invert making the holes in the connected components 255
    holes = cv2.bitwise_not(mask_floodfill[1:mask.shape[0] + 1,
                                           1:mask.shape[1] + 1])
    
    # fill the holes via OR and return
    return cv2.bitwise_or(mask, holes)


def is_max_in_neighborhood(image, radius):
    '''
    Determine if each pixel is the maximum in a neighborhood.
    
    image - one channel image
    radius - radius of the square used as a kernel
    '''
    
    with_border = cv2.copyMakeBorder(image, radius, radius,
                                     radius, radius, cv2.BORDER_CONSTANT,
                                     None, 0)
    
    collect = []
    
    for i in range(1, radius + 1):
        for j in range(1, radius + 1):
        
            collect.append(image > with_border[\
                                    radius + i : image.shape[0] + radius + i,
                                    radius + j : image.shape[1] + radius + j])
        
            collect.append(image > with_border[\
                                    radius - i : image.shape[0] + radius - i,
                                    radius - j : image.shape[1] + radius - j])
        
            collect.append(image > with_border[\
                                    radius - i : image.shape[0] + radius - i,
                                    radius + j : image.shape[1] + radius + j])
        
            collect.append(image > with_border[\
                                    radius + i : image.shape[0] + radius + i,
                                    radius - j : image.shape[1] + radius - j])

    mark = np.zeros(image.shape, dtype=np.uint8)
    
    mark[np.all(collect, axis=0)] = 255
    
    return mark