# -*- coding: utf-8 -*-
"""
Created on Tue Feb 27 17:13:51 2018

@author: Callie Darsey

I need to be able to
    1) View the boxplot of features
    2) Count the number of examples
"""

import os
import pickle
import numpy as np
import component_exploration_04 as fts
import matplotlib.pyplot as plt


'''
-------------------------------------------------------------------------------
Define the .data files holding the connected component information from
cv2.connectedComponentsWithStats() with appended elements for component target
and component features. The .data files hold the resized color frame, flow,
and exemplars as well.
-------------------------------------------------------------------------------
'''

# build up the paths (separated into the directory path and filename)
directory_paths = []
filenames = []

for tag_to_look_for in \
    ['way_1_cc_features_and_targets_from_frame_flow_exemplars__',
     'way_2_cc_features_and_targets_from_frame_flow_exemplars__',
     'way_3_cc_features_and_targets_from_frame_flow_exemplars__']:
        
    print('\n****************************************************************')
    print(tag_to_look_for)
    print('****************************************************************\n')
    
    # look for filenames beginning with 
    tag_len = len(tag_to_look_for)
    
    # folder to start from
    start_folder = os.path.join('.', 'video_copies_clips_track')
    
    # loop through folders and append
    for wave_folder in os.listdir(start_folder):
    
        dir_path_temp = os.path.join(start_folder, wave_folder)
        
        if os.path.isdir(dir_path_temp) == True:
        
            for first_identifying_folder in os.listdir(dir_path_temp):
                
                dir_path_temp = os.path.join(start_folder, wave_folder,
                                             first_identifying_folder)
                
                if os.path.isdir(dir_path_temp) == True:
                
                    for second_identifying_folder in os.listdir(dir_path_temp):
                        
                        dir_path_temp = os.path.join(start_folder, wave_folder,
                                                     first_identifying_folder,
                                                     second_identifying_folder)
                        
                        if os.path.isdir(dir_path_temp) == True:
                                        
                            for filename in os.listdir(dir_path_temp):
    
                                path = os.path.join(dir_path_temp, filename)                            
                                
                                if os.path.isfile(path) == True:
                                    
                                    if filename[0:tag_len] == tag_to_look_for:
                                        
                                        directory_paths.append(dir_path_temp)
                                        filenames.append(filename)
                                        
    '''
    Count the examples
    '''
    
    counting = {}
    number_of_keep_CCs_index = 0
    number_of_toss_CCs_index = 1
    
    keep_features = []
    toss_features = []
    
    for dir_path, filename in zip(directory_paths, filenames):
        
        path = os.path.join(dir_path, filename)
        
        with open(path, 'rb') as f:
            frame, flow, boxes, cc_info = pickle.load(f)
            
        features_stacked = np.vstack(cc_info[-1])
        targets_stacked = np.array(cc_info[-2]).reshape((-1, 1))
    
            
        keep_ind = np.where(targets_stacked == 1)[0]
        toss_ind = np.where(targets_stacked == 0)[0]
        
        if len(keep_features) == 0:
            keep_features = features_stacked[keep_ind, :]
        else:
            keep_features = np.vstack([keep_features,
                                       features_stacked[keep_ind, :]])
        
        if len(toss_features) == 0:
            toss_features = features_stacked[toss_ind, :]
        else:
            toss_features = np.vstack([toss_features,
                                       features_stacked[toss_ind, :]])
        
        ind = filename.rfind('__')
        identifying_part_of_name = filename[tag_len:ind]
        preceding, second_identifying_folder = os.path.split(dir_path)
        preceding, first_identifying_folder = os.path.split(preceding)
        
        identifier = os.path.join(first_identifying_folder,
                                  identifying_part_of_name)
        
        if  identifier not in counting.keys():
                
            counting[identifier] = [[], []]
            
        counting[identifier][number_of_keep_CCs_index].append(keep_ind.shape[0])
        counting[identifier][number_of_toss_CCs_index].append(toss_ind.shape[0])
    
    # print number of keep versus toss component examples
    print('\nNumber of Components Given Keep Targets:', keep_features.shape[0])
    print('\nNumber of Components Given Toss Targets:', toss_features.shape[0])
    
    # print number of clips
    print('\nNumber of clips:', len(counting.keys()))
    
    frames_per_clip = []
    
    keep_per_frame = []
    toss_per_frame = []
    
    keep_per_clip = []
    toss_per_clip = []
    
    for clip in counting.keys():
        
        # frames per clip
        frames_per_clip.append(len(counting[clip][number_of_keep_CCs_index]))
        
        # keep or toss per frame
        if len(keep_per_frame) == 0:
            keep_per_frame = np.array(counting[clip][number_of_keep_CCs_index])
        else:
            keep_per_frame = np.hstack([keep_per_frame,
                np.array(counting[clip][number_of_keep_CCs_index])])
            
        if len(toss_per_frame) == 0:
            toss_per_frame = np.array(counting[clip][number_of_toss_CCs_index])
        else:
            toss_per_frame = np.hstack([toss_per_frame,
                np.array(counting[clip][number_of_toss_CCs_index])])
        
        # keep or toss per clip
        if len(keep_per_clip) == 0:
            keep_per_clip = \
                np.array([np.sum(counting[clip][number_of_keep_CCs_index])])
        else:
            keep_per_clip = np.hstack([keep_per_clip,
                np.array([np.sum(counting[clip][number_of_keep_CCs_index])])])
            
        if len(toss_per_clip) == 0:
            toss_per_clip = \
                np.array([np.sum(counting[clip][number_of_toss_CCs_index])])
        else:
            toss_per_clip = np.hstack([toss_per_clip,
                np.array([np.sum(counting[clip][number_of_toss_CCs_index])])])
    
    print('\nFrames taken per clip:')
    print('\tMinimum', np.min(np.array(frames_per_clip)))
    print('\tMean', np.mean(np.array(frames_per_clip)))
    print('\tMedian', np.median(np.array(frames_per_clip)))
    print('\tMaximum', np.max(np.array(frames_per_clip)))
    
    print('\nkeep CCs per frame:\t\t\tToss CCs per frame:')
    print('\tMinimum {:d}\t\t\t\tMinimum {:d}'.format(np.min(keep_per_frame),
                                                      np.min(toss_per_frame)))
    print('\tMean {:.1f}\t\t\t\tMean {:.1f}'.format(np.mean(keep_per_frame),
                                                      np.mean(toss_per_frame)))
    print('\tMedian {:.1f}\t\t\t\tMedian {:.1f}'.format(np.median(keep_per_frame),
                                                      np.median(toss_per_frame)))
    print('\tMaximum {:d}\t\t\t\tMaximum {:d}'.format(np.max(keep_per_frame),
                                                      np.max(toss_per_frame)))
    
    print('\nkeep CCs per clip:\t\t\tToss CCs per clip:')
    print('\tMinimum {:d}\t\t\t\tMinimum {:d}'.format(np.min(keep_per_clip),
                                                      np.min(toss_per_clip)))
    print('\tMean {:.1f}\t\t\t\tMean {:.1f}'.format(np.mean(keep_per_clip),
                                                      np.mean(toss_per_clip)))
    print('\tMedian {:.1f}\t\t\t\tMedian {:.1f}'.format(np.median(keep_per_clip),
                                                      np.median(toss_per_clip)))
    print('\tMaximum {:d}\t\t\t\tMaximum {:d}'.format(np.max(keep_per_clip),
                                                      np.max(toss_per_clip)))
    
    
    # Display boxplots
    start_end_pairs = [[fts.FEATURE_B_HIST_LEFT, fts.FEATURE_B_HIST_RIGHT + 1],
                       [fts.FEATURE_G_HIST_LEFT, fts.FEATURE_G_HIST_RIGHT + 1],
                       [fts.FEATURE_R_HIST_LEFT, fts.FEATURE_R_HIST_RIGHT + 1],
                       [fts.FEATURE_H_HIST_LEFT, fts.FEATURE_H_HIST_RIGHT + 1],
                       [fts.FEATURE_S_HIST_LEFT, fts.FEATURE_S_HIST_RIGHT + 1],
                       [fts.FEATURE_V_HIST_LEFT, fts.FEATURE_V_HIST_RIGHT + 1],
                       [fts.FEATURE_Y_HIST_LEFT, fts.FEATURE_Y_HIST_RIGHT + 1],
                       [fts.FEATURE_Cr_HIST_LEFT, fts.FEATURE_Cr_HIST_RIGHT + 1],
                       [fts.FEATURE_Cb_HIST_LEFT, fts.FEATURE_Cb_HIST_RIGHT + 1],
                       
                       [fts.FEATURE_B_PDF_LEFT, fts.FEATURE_B_PDF_RIGHT + 1],
                       [fts.FEATURE_G_PDF_LEFT, fts.FEATURE_G_PDF_RIGHT + 1],
                       [fts.FEATURE_R_PDF_LEFT, fts.FEATURE_R_PDF_RIGHT + 1],
                       [fts.FEATURE_H_PDF_LEFT, fts.FEATURE_H_PDF_RIGHT + 1],
                       [fts.FEATURE_S_PDF_LEFT, fts.FEATURE_S_PDF_RIGHT + 1],
                       [fts.FEATURE_V_PDF_LEFT, fts.FEATURE_V_PDF_RIGHT + 1],
                       [fts.FEATURE_Y_PDF_LEFT, fts.FEATURE_Y_PDF_RIGHT + 1],
                       [fts.FEATURE_Cr_PDF_LEFT, fts.FEATURE_Cr_PDF_RIGHT + 1],
                       [fts.FEATURE_Cb_PDF_LEFT, fts.FEATURE_Cb_PDF_RIGHT + 1],
                       
                       [fts.FEATURE_DFT_MAG_SPEC_HIST_LEFT,
                        fts.FEATURE_DFT_MAG_SPEC_HIST_RIGHT + 1],
                       [fts.FEATURE_DFT_PHASE_SPEC_HIST_LEFT,
                        fts.FEATURE_DFT_PHASE_SPEC_HIST_RIGHT + 1],
                       [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_LEFT,
                        fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_RIGHT],
                     [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT,
                      fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT + 1],
                     [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT,
                      fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT + 1]]
                     
    start_end_titles = ['Blue Histogram',
                        'Green Histogram',
                        'Red Histogram',
                        'Hue Histogram',
                        'Saturation Histogram',
                        'Value Histogram',
                        'Luma Histogram',
                        'Red-difference Chroma Component Histogram',
                        'Blue-difference Chroma Component Histogram',
                        
                        'Blue PDF',
                        'Green PDF',
                        'Red PDF',
                        'Hue PDF',
                        'Saturation PDF',
                        'Value PDF',
                        'Luma PDF',
                        'Red-difference Chroma Component PDF',
                        'Blue-difference Chroma Component PDF',
                        
                        'DFT Magnitude Spectrum Histogram',
                        'DFT Phase Spectrum Histogram',
                        'Motion histogram',
                        'Motion histogram normalized by count',
                        'Motion histogram normalized by area']
    
    single_indices = [fts.FEATURE_CANNY_EDGE_TO_AREA_RATIO,
                      fts.FEATURE_AREA,
                      fts.FEATURE_PERIMETER,
                      fts.FEATURE_MIN_AREA_RECT_ORIENT,
                      fts.FEATURE_AREA_TO_MIN_AREA_RECT,
                      fts.FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO]
    
    single_titles = ['Canny edge to area ratio',
                     'Area',
                     'Perimeter',
                     'Orientation of Minimum Area Bounding Box',
                     'Area to Minimum Area Bounding Box Area',
                     'Short to Long Sides Ratio of Minimum Area Bounding Box']
    
    for title, start_end in zip(start_end_titles, start_end_pairs):
    
        print('\n', title)
        
        # keep
        f_keep = plt.figure(figsize=(21, 4))
        plt.boxplot(keep_features[:, start_end[0] : start_end[1]], sym='m+')
        plt.title('keep')
        a_keep = f_keep.gca()
        a_keep_vals = a_keep.axis()
        
        # toss
        f_toss = plt.figure(figsize=(21, 4))
        plt.boxplot(toss_features[:, start_end[0] : start_end[1]], sym='m+')
        plt.title('toss')
        a_toss = f_toss.gca()
        a_toss_vals = a_toss.axis()
        
        # make both plots have the same axis
        a_vals = [min([a_keep_vals[0], a_toss_vals[0]]),
                  max([a_keep_vals[1], a_toss_vals[1]]),
                  min([a_keep_vals[2], a_toss_vals[2]]),
                  max([a_keep_vals[3], a_toss_vals[3]])]
        
        a_keep.axis(a_vals)
        a_toss.axis(a_vals)
        
        plt.show()
    
    
        
    for title, index in zip(single_titles, single_indices):
    
        print('\n', title)
        
        # keep
        f_keep = plt.figure(figsize=(4, 3))
        plt.boxplot(keep_features[:, index], sym='m+')
        plt.title('keep')
        a_keep = f_keep.gca()
        a_keep_vals = a_keep.axis()
        
        # toss
        f_toss = plt.figure(figsize=(4, 3))
        plt.boxplot(toss_features[:, index], sym='m+')
        plt.title('toss')
        a_toss = f_toss.gca()
        a_toss_vals = a_toss.axis()
        
        # make both plots have the same axis
        a_vals = [min([a_keep_vals[0], a_toss_vals[0]]),
                  max([a_keep_vals[1], a_toss_vals[1]]),
                  min([a_keep_vals[2], a_toss_vals[2]]),
                  max([a_keep_vals[3], a_toss_vals[3]])]
        
        a_keep.axis(a_vals)
        a_toss.axis(a_vals)
        
        plt.show() 


