# -*- coding: utf-8 -*-
"""
Created on Fri Feb  2 12:23:36 2018

@author: Callie Darsey

OpenCV Python tutorial example

http://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_imgproc/
py_watershed/py_watershed.html
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt

img = cv2.imread('water_coins.jpg')
gray = cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
ret, thresh = cv2.threshold(gray,0,255,cv2.THRESH_BINARY_INV+cv2.THRESH_OTSU)

# noise removal
kernel = np.ones((3,3),np.uint8)
opening = cv2.morphologyEx(thresh, cv2.MORPH_OPEN, kernel, iterations = 2)

# sure background area
sure_bg = cv2.dilate(opening, kernel, iterations=3)

# Finding sure foreground area
'''
dist_transform = cv2.distanceTransform(opening, distanceType=cv2.DIST_L2,
                                       maskSize=5)
'''
# replacing with precise
dist_transform = cv2.distanceTransform(opening, distanceType=cv2.DIST_L2,
                                       maskSize=cv2.DIST_MASK_PRECISE)
threshold = 0.7*dist_transform.max()
maxval = 255
ret, sure_fg = cv2.threshold(dist_transform, threshold, maxval,
                             cv2.THRESH_BINARY)

# Finding unknown region
sure_fg_uint8 = np.uint8(sure_fg)
unknown = cv2.subtract(sure_bg, sure_fg_uint8)


# Marker labelling
ret, markers = cv2.connectedComponents(sure_fg_uint8)
print('markers', markers.dtype)

# Add one to all labels so that sure background is not 0, but 1
markers = markers+1

# Now, mark the region of unknown with zero
markers[unknown==255] = 0

# get the watershed markers
plt.matshow(markers)
markers = cv2.watershed(img, markers)
print('markers', markers.dtype)

# make the -1 watershed boundary with blue in the image
img[markers == -1] = [255,0,0]

#
m = np.copy(markers)
m[m == -1] = -2

watershed_lines = np.zeros(markers.shape, dtype=np.uint8)
watershed_lines[markers == -1] = 255

# display
cv2.imshow('coins', watershed_lines)
cv2.waitKey(0)
cv2.destroyAllWindows()