# -*- coding: utf-8 -*-
"""
Created on Sun May 27 17:22:25 2018

@author: Callie Darsey

establish_skin_detectors_01
"""

import skin_detector_Abby_01
import attempt_log_reg_AND_knn_with_Cleanup_01
import skin_detector_01
import numpy as np
import pickle
import os
import cv2
import patches_01 as patches
import sklearn.linear_model as sklm
import sklearn.neighbors as skn
import sklearn.ensemble as ske
import sklearn.model_selection as skm
import sklearn.metrics as skmet
import matplotlib.pyplot as plt



if __name__ == '__main__':
        
    # paths
    path = './video_copies_clips_track/ROI_masks_patch_detection/'
    feature_vectors = path + "block_feature_vectors/"
    out_path = '../WriteUp/gathering_01/skin_detector/'
    
    filenames = [\
    'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
    'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
    'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
    'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
    'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
    'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
    'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
    'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
    'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
    'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
    'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
    'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
    'STI WA May09 B-MVI_0006_trans__600_700_0496.data']    
    
    filenames = np.array(filenames)
    indices = np.arange(len(filenames))
    
    print('number of frames:', len(filenames))
    
    #for block_dim in [2, 3, 4, 5]:
    #for block_dim in [6, 7, 8]:
    for block_dim in [2]:
        
        print('block size {:d} x {:d}'.format(block_dim, block_dim))
        
        collect_val_recall = []
        collect_val_precision = []
    
        for i in indices:
            
            val_test_filename = filenames[i]
            train_filenames = filenames[np.where(indices != i)[0]]
            
            # get the feature vectors
            
            hand_feature_vectors_collect = []
            non_skin_feature_vectors_collect = []       
            
            for filename in train_filenames:
                
                print(filename)
            
                with open(feature_vectors + filename[0:-5] + \
                          '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                    
                    hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
                # I assume there are more non-skin than hand, so verify
                assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
            
                # shuffle, then take a subset of non-skin
                np.random.shuffle(non_skin_feature_vectors)
                
                non_skin_feature_vectors_take = \
                    non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
                
                # save for further stacking
                hand_feature_vectors_collect.append(hand_feature_vectors)
                non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
                
            feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
            feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
            
            # hand 1 and non-skin 0
            y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                           np.zeros(feature_vectors_non_skin.shape[0])])
            
            # feature vectors
            X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
            
            # shuffle
            indices_shuffled = np.arange(0, y.shape[0])
            np.random.shuffle(indices_shuffled)
            
            y = y[indices_shuffled]
            X = X[indices_shuffled, :]
            print('X.shape', X.shape)
    
            # train knn
            clf_knn_default = skn.KNeighborsClassifier()
            clf_knn_default.fit(X, y)
            
            # train logistic regression
            clf_log_reg_default = sklm.LogisticRegression()
            clf_log_reg_default.fit(X, y)
            
            # validation test
        
            print('validation test filename:', val_test_filename)
            
            # associated with ROIs
            with open(feature_vectors + val_test_filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
            y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                           np.zeros(non_skin_feature_vectors.shape[0])])
            
            X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
            
            y_predicted_by_knn = clf_knn_default.predict(X_ROI)
            y_predicted_by_log_reg = clf_log_reg_default.predict(X_ROI)
            
            y_predicted = np.all([y_predicted_by_knn,
                                  y_predicted_by_log_reg], axis=0)
            
            recall = skmet.recall_score(y_ROI, y_predicted,
                                        labels=[0, 1], average=None)
            
            precision = skmet.precision_score(y_ROI, y_predicted)
        
            collect_val_recall.append(recall)
            collect_val_precision.append(precision)  
            
            print('confusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
            print('precision:       ', precision)
            print('sensitivity:     ', recall[1])
            print('specificity:     ', recall[0])
            
            
            # not associated with ROIs
            with open(path + val_test_filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
            
            X = patches.get_valid_blocks_as_feature_vectors(frame, \
                np.ones(frame.shape[0:2]), block_dim)
            
            predicted_labels_by_knn = clf_knn_default.predict(X)
            predicted_labels_by_log_reg = clf_log_reg_default.predict(X)
            
            predicted_labels = np.all([predicted_labels_by_knn,
                                       predicted_labels_by_log_reg], axis=0)


            skin_mask_knn = \
                patches.build_mask_from_patch_predictions(frame,
                                                    predicted_labels_by_knn,
                                                    block_dim)                
            skin_mask_knn_display = cv2.bitwise_and(frame, frame,
                                                    mask=skin_mask_knn)


            skin_mask_log_reg = \
                patches.build_mask_from_patch_predictions(frame,
                                                predicted_labels_by_log_reg,
                                                          block_dim)                
            skin_mask_log_reg_display = cv2.bitwise_and(frame, frame,
                                                mask=skin_mask_log_reg)

            
            skin_mask_knn_log_reg = \
                patches.build_mask_from_patch_predictions(frame,
                                                          predicted_labels,
                                                          block_dim)                
            skin_mask_knn_log_reg_display = cv2.bitwise_and(frame, frame,
                                                mask=skin_mask_knn_log_reg)
        
    
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__knn__block_dim_{:02d}'.format(block_dim) + \
                        '.png', skin_mask_knn_display)
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__log_reg__block_dim_{:02d}'.format(block_dim) + \
                        '.png', skin_mask_log_reg_display)
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__knn_and_log_reg__block_dim_{:02d}'.format(\
                        block_dim) + '.png', skin_mask_knn_log_reg_display)
            
            
            cleaned_mask = attempt_log_reg_AND_knn_with_Cleanup_01.cleanup(\
                skin_mask_knn_log_reg)
        
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__knn_and_log_reg__block_dim_02-cleaned.png',
                        cv2.bitwise_and(frame, frame, mask=cleaned_mask))            

            abby_no_cleaning_mask = \
                skin_detector_Abby_01.DetectSkinRegions(frame)
                
            abby_cleaned = \
                skin_detector_Abby_01.PersonObject(frame, show=False)
                
            shi_mask = skin_detector_01.DetectSkinRegions(frame)
            
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__Abby_no_cleaning'.format(block_dim) + '.png',
                    cv2.bitwise_and(frame, frame, mask=abby_no_cleaning_mask))
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__Abby_post_cleaning'.format(block_dim) + \
                        '.png', abby_cleaned)
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__Shi_Thesis'.format(block_dim) + '.png',
                        cv2.bitwise_and(frame, frame, mask=shi_mask))            

            hand_mask = np.any([m for i, m in obj_data.items() \
                                      if i[0:4] == 'hand'], axis=0)
        
            non_skin_mask = np.any([m for i, m in obj_data.items() \
                                          if i[0:8] == 'non_skin'], axis=0)


            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__hand'.format(block_dim) + '.png',
                        cv2.bitwise_and(frame, frame,
                                        mask=hand_mask.astype(np.uint8)))
            
            cv2.imwrite(out_path + val_test_filename[0:-5] + '/' + \
                        val_test_filename[0:-5] + \
                        '__non_skin'.format(block_dim) + '.png',
                        cv2.bitwise_and(frame, frame,
                                        mask=non_skin_mask.astype(np.uint8)))  
            
        print('Precision  :',
              'min {:.2f}, median {:.2f}, mean {:.2f}, max {:.2f}'.format(\
                   np.min(collect_val_precision),
                   np.median(collect_val_precision),
                   np.mean(collect_val_precision),
                   np.max(collect_val_precision)))    
        
        collect_val_recall = np.vstack(collect_val_recall)
    
        print('Sensitivity:',
              'min {:.2f}, median {:.2f}, mean {:.2f}, max {:.2f}'.format(\
                   np.min(collect_val_recall[:, 1]),
                   np.median(collect_val_recall[:, 1]),
                   np.mean(collect_val_recall[:, 1]),
                   np.max(collect_val_recall[:, 1])))
        
        print('Specificity:',
              'min {:.2f}, median {:.2f}, mean {:.2f}, max {:.2f}'.format(\
                   np.min(collect_val_recall[:, 0]),
                   np.median(collect_val_recall[:, 0]),
                   np.mean(collect_val_recall[:, 0]),
                   np.max(collect_val_recall[:, 0])))

        
        header          = \
        '              mean   std    min     Q1   median    Q3    max    IQR '
        precision_row   = \
        'Precision    {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:6.3f}  {:.3f}  {:.3f}  {:.3f}'
        sensitivity_row = \
        'Sensitivity  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:6.3f}  {:.3f}  {:.3f}  {:.3f}'
        specificity_row = \
        'Specificity  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:6.3f}  {:.3f}  {:.3f}  {:.3f}'

        
        print(header)


        collect_val_precision = np.array(collect_val_precision)
        collect_val_precision.sort()
        num = len(collect_val_precision)            
        print(precision_row.format(\
            np.mean(collect_val_precision),
            np.std(collect_val_precision),
            np.min(collect_val_precision),
            np.median(collect_val_precision[0:int(num / 2)]),
            np.median(collect_val_precision),
            np.median(collect_val_precision[-int(num / 2):]),
            np.max(collect_val_precision),
            np.median(collect_val_precision[-int(num / 2):]) - \
            np.median(collect_val_precision[0:int(num / 2)])))


        collect_val_recall[:, 1].sort()
        num = len(collect_val_recall[:, 1])        
        print(sensitivity_row.format(\
            np.mean(collect_val_recall[:, 1]),
            np.std(collect_val_recall[:, 1]),
            np.min(collect_val_recall[:, 1]),
            np.median(collect_val_recall[0:int(num / 2), 1]),
            np.median(collect_val_recall[:, 1]),
            np.median(collect_val_recall[-int(num / 2):, 1]),
            np.max(collect_val_recall[:, 1]),
            np.median(collect_val_recall[-int(num / 2):, 1]) - \
            np.median(collect_val_recall[0:int(num / 2), 1])))

        collect_val_recall[:, 0].sort()
        num = len(collect_val_recall[:, 0])        
        print(specificity_row.format(\
            np.mean(collect_val_recall[:, 0]),
            np.std(collect_val_recall[:, 0]),
            np.min(collect_val_recall[:, 0]),
            np.median(collect_val_recall[0:int(num / 2), 0]),
            np.median(collect_val_recall[:, 0]),
            np.median(collect_val_recall[-int(num / 2):, 0]),
            np.max(collect_val_recall[:, 0]),
            np.median(collect_val_recall[-int(num / 2):, 0]) - \
            np.median(collect_val_recall[0:int(num / 2), 0])))
        



        