# -*- coding: utf-8 -*-
"""
Created on Sat Apr 14 22:03:22 2018

@author: Callie Darsey

save feature vectors for different patch sizes
"""

import os
import pickle
import patches_01 as patches
import numpy as np

# path
path = './video_copies_clips_track/ROI_masks_patch_detection/'

# get list .data files that I got via the select_roi_and_pickle_tool_01.py tool
[f for f in os.listdir(path) if os.path.isfile(path + f)]
filenames = [f for f in os.listdir(path) if os.path.isfile(path + f)]

# make the review folder
save = path + "block_feature_vectors/"
if not os.path.isdir(save):
    os.mkdir(save)


print('Number of frames: {}\n'.format(len(filenames)))

for block_dim in [2, 3, 4, 5]:

    print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
    
    hand_feature_vectors_collect = []
    non_skin_feature_vectors_collect = []
    
    for filename in filenames:
        
        print(filename)
        
        with open(path + filename, 'rb') as f:
            
            roi_masks, frame = pickle.load(f)
            
        hand_mask_collect = []
        non_skin_mask_collect = []
        
        for key in roi_masks.keys():
            
            if 'hand' in key:
                
                hand_mask_collect.append(roi_masks[key])
                
            elif 'non_skin' in key:
                
                non_skin_mask_collect.append(roi_masks[key])
                
        hand_mask = np.any(hand_mask_collect, axis=0)       
        non_skin_mask = np.any(non_skin_mask_collect, axis=0)
        
        hand_feature_vectors = \
            patches.get_valid_blocks_as_feature_vectors(frame, hand_mask,
                                                        block_dim)
    
        non_skin_feature_vectors = \
            patches.get_valid_blocks_as_feature_vectors(frame, non_skin_mask,
                                                        block_dim)
        print('Number of hand feature vector:     {}'.format(\
              hand_feature_vectors.shape[0]))
        print('Number of non-skin feature vector: {}\n'.format(\
              non_skin_feature_vectors.shape[0]))
    
        with open(save+filename[0:-5] + \
                  '_block_dim_{:02d}.data'.format(block_dim), 'wb') as f:
            
            pickle.dump((hand_feature_vectors, non_skin_feature_vectors), f)
