# -*- coding: utf-8 -*-
"""
Created on Thu Apr 19 11:20:28 2018

@author: Callie Darsey

First attempt at drawing bounding boxes after filling holes and opening
the union over 10 thresholded flow frames.

"""

import cv2
import pickle
import numpy as np
import os
import component_functions_01 as comp_fns
import fns_investigating_optical_flow_01 as fns

def locate_window(title):
    '''
    This function places the window for openCV
    
    '''
    
    move_window = [0, -30]
    cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1]) 



# specify paths to where the frame and flow are pickled
common = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
videoname = 'MVI_0028_trans__200_208.mp4'

''' We have the pickled frame and flow have names in the form
    flow_<video clip name>_<####>_<####>.data, and those frames
    have 3 frames dropped in between frames where flow is calculated.
'''
framesToAdvance = 4

dir_cont = os.listdir(common + specific)
dir_keep = []
for cont in dir_cont:
    if cont[0:5 + len(videoname[0:-4])] == 'flow_' + videoname[0:-4]:
        dir_keep.append(cont)
dir_keep.sort()
        
beg_frame = dir_keep[0][-14:-10]
end_frame = dir_keep[len(dir_keep)-1][-9:-5]

# number of frames to take union over flow mag. thresholded
union_num = 10

# threshold on flow mag. thresholded
thresh = 7.0

# collect union maps
union_maps = {}

# prep for first segment
seg_fn = int(beg_frame)

thresh_flow_mag_collect = []
flow_mag_collect = []
Y_collect = []


for fn in range(int(beg_frame), int(end_frame), framesToAdvance):
    
    if (fn + int(beg_frame)) % (union_num * framesToAdvance) == 0 or \
       fn == int(end_frame) - framesToAdvance:
        
        if len(thresh_flow_mag_collect):
            
            # union mask
            union_mask = \
                np.any(thresh_flow_mag_collect, axis=0).astype(np.uint8) * 255
                
            # fill holes
            union_map = comp_fns.fill_holes(union_mask)


            '''don't deal with unions near the border'''
            # boarder mask
            border = 20

            border_mask = np.ones_like(union_map)
            border_mask[border:border_mask.shape[0] - border,
                        border:border_mask.shape[1] - border] = 0

            # get connected component info
            cc_union_map = cv2.connectedComponentsWithStats(union_map)

            # flag for removal
            remove = []
            for label in range(0, cc_union_map[0]):
                
                label_matrix = cc_union_map[1] == label
                
                # take action when cc overlaps border
                overlap = np.count_nonzero(np.all([label_matrix, border_mask],
                                                  axis=0))
                if overlap > 0:
                    
                   # over laps border completely
                   if overlap == cc_union_map[2][label, cv2.CC_STAT_AREA]:
                       remove.append(label)
                       
                   # overlaps partially
                   elif overlap > 0.25 * cc_union_map[2][label,
                                                     cv2.CC_STAT_AREA]:
                       remove.append(label)

            if len(remove) > 0:           
                r, c = np.where(np.any([cc_union_map[1] == label \
                                        for label in remove], axis=0))
                cc_union_map[1][r, c] = 0            


            '''Don't deal with too small areas'''                        
            # remove the small areas
            too_small = \
                np.where(cc_union_map[2][:, cv2.CC_STAT_AREA] < 200)[0]
                
            zero_mask = np.any([cc_union_map[1] == label \
                                for label in too_small], axis=0)
            
            cc_union_map[1][zero_mask] = 0

            '''Save the remaining components'''
            # re-get the connected components
            union_map = cc_union_map[1].astype(np.bool).astype(np.uint8)
            cc_union_map = cv2.connectedComponentsWithStats(union_map)

            # save the union map
            union_maps[(seg_fn , fn)] = cc_union_map
            
        # prep for next segment
        seg_fn = fn
        
        thresh_flow_mag_collect = []
        flow_mag_collect = []
        Y_collect = []

    else:
        
        fn1 = '{:04d}'.format(fn)
        fn2 = '{:04d}'.format(fn + framesToAdvance)        

        # load the frame and flow
        with open(common + specific + 'flow_' + videoname[0:-4] + \
                  '_{}_{}.data'.format(fn1, fn2), 'rb') as f:
            
            p = pickle.load(f)
            
        frame1, frame2, flow = p[0], p[1], p[2]
    
        # flow magnitude
        flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])

        # collect the thresholded flow
        thresh_flow_mag_collect.append(flow_mag > thresh)
        


for first, last in union_maps.keys():
    
    for fn in np.arange(first, last + framesToAdvance, framesToAdvance):
        
        fn1 = '{:04d}'.format(fn)
        fn2 = '{:04d}'.format(fn + framesToAdvance)        

        # load the frame and flow
        with open(common + specific + 'flow_' + videoname[0:-4] + \
                  '_{}_{}.data'.format(fn1, fn2), 'rb') as f:
            
            p = pickle.load(f)
            
        frame1 = p[0]
        flow = p[2]

        # make frame to display with boxes
        arrows = np.copy(frame1)    
        arrows = fns.draw_flow(arrows, flow,
                               np.ones(frame1.shape[0:2], dtype=np.uint8),
                               step=4)
        
        for frame in [frame1, arrows]:

            # draw the text
            cv2.putText(frame, str(fn), (frame.shape[1] - 100, 25),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
            
            for label in range(0, union_maps[(first, last)][0]):
                
                # box
                top, height, left, width = \
                    union_maps[(first, last)][2][label, cv2.CC_STAT_TOP],\
                    union_maps[(first, last)][2][label, cv2.CC_STAT_HEIGHT],\
                    union_maps[(first, last)][2][label, cv2.CC_STAT_LEFT],\
                    union_maps[(first, last)][2][label, cv2.CC_STAT_WIDTH]
                    
                # draw box
                cv2.rectangle(frame, (left, top),
                              (left + width - 1, top + height - 1),
                              (0, 255, 255), 2)


        title = 'title'; locate_window(title); cv2.imshow(title, arrows)
        cv2.waitKey(6)
            
cv2.destroyAllWindows()        
        
        
        
        
        
        
        
