# -*- coding: utf-8 -*-
"""
Created on Thu Mar 22 12:15:26 2018

@author: Callie Darsey


grid search for way 1

"""


import component_exploration_04 as fts
import pickle
import numpy as np
import sklearn.ensemble as sk_ensemble
import sklearn.model_selection as sk_ms
import cv2
import datetime

def function():
        
    e1 = cv2.getTickCount()
    
    file = 'way_1_cc_features.data'
        
    print('\n\n', file, '\n')
    
    # load the pickled features
    with open(file, 'rb') as f:
        keep_features, toss_features = pickle.load(f)
    
    
    # take out some features
    collect_keep_features_to_use = []
    collect_toss_features_to_use = []
    
    
    starts = [fts.FEATURE_H_PDF_LEFT,
              fts.FEATURE_S_PDF_LEFT,
              fts.FEATURE_V_PDF_LEFT]
    
    ends = [fts.FEATURE_H_PDF_RIGHT,
            fts.FEATURE_S_PDF_RIGHT,
            fts.FEATURE_V_PDF_RIGHT]
    
    combines = [10, 16, 16]
    for start, end, combine in zip(starts, ends, combines):
        
        for s in range(start, end + 1, combine):
            
            combined = np.sum(keep_features[:, s : s + combine], axis=1)
            combined = combined.reshape((-1, 1))
    
            collect_keep_features_to_use.append(combined)
            
            combined = np.sum(toss_features[:, s : s + combine], axis=1)
            combined = combined.reshape((-1, 1))
    
            collect_toss_features_to_use.append(combined)
    
    print('flow normalized by bin count')
    
    starts = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT]
    
    ends = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT]  
    
    
    for start, end in zip(starts, ends):
        
        collect_keep_features_to_use.append(keep_features[:, start : end + 1])
        collect_toss_features_to_use.append(toss_features[:, start : end + 1])
        
        
    for index in [fts.FEATURE_CANNY_EDGE_TO_AREA_RATIO,
                  fts.FEATURE_AREA,
                  fts.FEATURE_PERIMETER,
                  fts.FEATURE_MIN_AREA_RECT_ORIENT,
                  fts.FEATURE_AREA_TO_MIN_AREA_RECT,
                  fts.FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO]:
        
        column = keep_features[:, index].reshape((-1, 1))
        
        collect_keep_features_to_use.append(column)
        
        column = toss_features[:, index].reshape((-1, 1))
        
        collect_toss_features_to_use.append(column)           
     
    # use
    keep_features_used = np.hstack(collect_keep_features_to_use)
    toss_features_used = np.hstack(collect_toss_features_to_use)
    
    
        
    # print the training set size
    n_keep_samples, n_keep_features = keep_features_used.shape
    n_toss_samples, n_toss_features = toss_features_used.shape
    
    assert n_keep_features == n_toss_features
    
    n_features = n_keep_features
    
    print('Number of features: {}'.format(n_features))
    print('Number of keep samples available: {}'.format(n_keep_samples))
    print('Number of toss samples available: {}'.format(n_toss_samples))
    
    
    '''
    Force the class with more samples to have just as many samples by randomly
    choosing a subset.
    '''
    if n_keep_samples > n_toss_samples:
        
        np.random.shuffle(keep_features_used)
        
        X = np.vstack([keep_features_used[0:n_toss_samples, :],
                       toss_features_used])
    
        y = np.hstack([np.ones(n_toss_samples, dtype=np.int),
                       np.zeros(n_toss_samples, dtype=np.int)])
    
    elif n_keep_samples < n_toss_samples:
        
        np.random.shuffle(toss_features_used)
        
        X = np.vstack([keep_features_used,
                       toss_features_used[0:n_keep_samples, :]])
    
        y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                       np.zeros(n_keep_samples, dtype=np.int)])
    
    else:
    
        X = np.vstack([keep_features_used, toss_features_used])
    
        y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                       np.zeros(n_toss_samples, dtype=np.int)])
    
    # shuffle the training set and targets
    n_samples = X.shape[0]
    
    shuffled_indices = np.arange(n_samples)
    np.random.shuffle(shuffled_indices)
    
    X = X[shuffled_indices, :]
    y = y[shuffled_indices]
    
    
    rf_clf = sk_ensemble.RandomForestClassifier()
    
    param_distributions = {'n_estimators' : [10, 50, 100, 200, 500, 1000],
                           'criterion' : ['gini'],
                  
                           'max_features' : ['sqrt', 'log2', None, 1, 4, 0.25],
                           'max_depth' : [None, 5, 50, 190],
                  
                           'min_samples_split' : [2, 10, 100],
                           'min_samples_leaf' : [1, 20, 400],
                           'min_weight_fraction_leaf' : [0.0, 0.01, 0.1],
    
                           'max_leaf_nodes' : [None, 70, 10, 2],
                  
                           'min_impurity_split' : [0.0000001,
                                                   0.00001,
                                                   0.001],
                           'bootstrap' : [True],
                           'oob_score' : [True],
                           'n_jobs' : [-1],
                           'random_state' : [None],
                           'verbose' : [1],
                           'warm_start' : [False],
                           'class_weight' : [None, 'balanced_subsample']}    
    
    combinations = 1
    for lst in param_distributions.values():
        combinations = combinations * len(lst)
    print('combinations:', combinations)
    
    # grid search
    n_iter = 9331
    print('n_iter:', n_iter)
    gs_rf_clf = sk_ms.RandomizedSearchCV(rf_clf, param_distributions,
                                         n_iter=n_iter, scoring='f1',
                                         fit_params=None, n_jobs=1, iid=True,
                                         refit=True, cv=None, verbose=10,
                                         pre_dispatch=None,
                                         random_state=None,
                                         error_score='raise',
                                         return_train_score=True)
                                         
    gs_rf_clf.fit(X, y)
                                         
    # best estimator
    print(gs_rf_clf.best_estimator_)
    
    # look at the out-of-bag scores
    print('out-of-bag accuracy score: {:.4f}'.format( \
          gs_rf_clf.best_estimator_.oob_score_))
    
    # pickle result
    n = datetime.datetime(1, 1, 1).now()
    timestamp = 'y{:d}m{:02d}d{:02d}-h{:02d}m{:02d}s{:02d}.{:d}'. \
        format(n.year, n.month, n.day,
               n.hour, n.minute, n.second, n.microsecond)
    pickle_path = 'way_1_gs_rf_clf_' + str(n_features) + timestamp + '.data'
    with open(pickle_path, 'wb') as f:
        pickle.dump((gs_rf_clf), f)
                
        
    
    e2 = cv2.getTickCount()
    time = (e2 - e1)/ cv2.getTickFrequency()
    print("Execution time in seconds = ", time, "\n")
    
