# -*- coding: utf-8 -*-
"""
Created on Thu Feb  1 14:39:37 2018

@author: Callie Darsey (cj.darsey@gmail.com)

This an initial attempt at forearm detection in a region of motion.

    1) Acknowledge a region of motion after thresholding
    2) Look for lines in or near the region of motion that are approximately
       parallel.
    3) When there are approximately parallel lines in or near region of motion,
       see if one or both ends there is skin detected. If so, then this is
       potentially a forearem and hand that we'd want to track and collect
       motion vector info over.
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import skin_detector_01 as skin_detector
import os

# video path
path_common = './video_copies_clips_track/copies_and_clips/'
path_specific = '/STI WA Apr25 A/STI WA Apr25 AMVI_0068_trans/'
video_name = 'STI WA Apr25 AMVI_0068_trans__860_910.mp4'
video_path = path_common + path_specific + video_name
             

# Setting to play with

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = 4

'''Resolution rescaling factor'''
scaleFactor = 0.7

'''
normalization of optical flow by resolution scaling factor and 
number of frames advanced
'''
normalize_scale = True
normalize_frame_advance = False

''' wait to calculate optical flow and display '''
frame_wait = 0

'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''

pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0


'''
write out a video
'''
video_write = False

'''
write out images
'''
image_write = True

# other settings
displayVideo = True
threshold = 0.0
      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2. CAP_PROP_FPS )
print('number of frames:', num_frames, ', frames per second:', fps)



# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

# prep window
title = 'display window'
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
#cv2.moveWindow(title, 0, -20)
#cv2.moveWindow(title, -468, -1111)
cv2.moveWindow(title, -234, -900)

# loop through the frames and display

# announce
print("Press Esc on the image to exit")

e1 = cv2.getTickCount()

count = 0


# for when write_video is true
if video_write == True:
    
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    root, ext = os.path.splitext(video_path)
    video_path_less_frames = root + '__skipped_' + str(framesToAdvance - 1) + ext
    out = cv2.VideoWriter(video_path_less_frames, fourcc, 20, 
                          tuple(np.flip(frame1_full.shape[0:2], 0)))
    
    if out.isOpened():
        out.write(frame1_full)
    else:
        print('\nout not open\n')
        
# for when image_write is true
if image_write == True:
    
    image_path = path_common + path_specific + 'images/' + \
        video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
    
    cv2.imwrite(image_path, frame1)

while(cap.isOpened()):
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            break
     
    if success_flag:
    # process upon true read frame success_flag
    
        if video_write:
    
            if out.isOpened():
                out.write(frame2_full)
            else:
                print('\nout not open\n')
                
        #initialize frame2
        frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
        frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)

        if image_write == True:    
            
            image_path = path_common + path_specific + 'images/' + \
            video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
            
            cv2.imwrite(image_path, frame2)
        
        
        if count > frame_wait:
            # calculate the flow between frame1 and frame2    
            flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                flow=None,
                                                pyr_scale=pyr_scale, levels=levels,
                                                winsize=winsize,
                                                iterations=iterations,
                                                poly_n=poly_n,
                                                poly_sigma=poly_sigma, flags=flags)
            
            mag = np.sqrt(flow[...,0]**2 + flow[..., 1]**2)
            
            bp1 = plt.boxplot(np.reshape(mag, -1))
            plt.show()
            
            mask1 = mag > bp1['whiskers'][1].get_data()[1][1]
            
            bp2 = plt.boxplot(mag[mask1])
            plt.show()
            
            mask = (mag > bp2['medians'][0].get_data()[1][0]).astype(np.uint8)
            
            kernel = np.ones((5,5),np.uint8)
            
            mask = cv2.morphologyEx(mask, cv2.MORPH_OPEN, kernel)
            mask = cv2.dilate(mask, kernel)
            
            
            
            
        if displayVideo and count > frame_wait:

            if normalize_scale and normalize_frame_advance:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow * (1 / scaleFactor) / framesToAdvance,
                                       mask, step=4)
            elif normalize_scale:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow * (1 / scaleFactor),
                                       mask, step=4)
            elif normalize_frame_advance:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow / framesToAdvance,
                                       mask, step=4)
            else:                
                arrows = fns.draw_flow(frame1_gray, flow,
                                       mask, step=4)                
            
            cv2.imshow(title, arrows)
            #cv2.moveWindow(title, 0, 0)

        # update frame1
        frame1_gray = frame2_gray
                
        # Esc key press will escape
        k = cv2.waitKey(1) & 0xff
        if k == 27:

            # release capture
            cap.release()

            # close all the OpenCV windows
            cv2.destroyAllWindows()
            
            break
    else:

        # release capture
        cap.release()

        # close all the OpenCV windows
        cv2.destroyAllWindows()

if video_write == True:
    
    out.release()

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)
