# -*- coding: utf-8 -*-
"""
Created on Wed May  2 18:16:06 2018

@author: Callie Darsey

initial_patch_training_log_reg_01
"""


import pickle
import os
import numpy as np
import sklearn.linear_model as sklm
import sklearn.model_selection as skm
import sklearn.metrics as skmet

path = './video_copies_clips_track/ROI_masks_patch_detection/'
save = path + "block_feature_vectors/"
#save = path + "block_feature_vectors_non_skin_misclassified_patches/"

roi_path = './video_copies_clips_track/ROI_masks_patch_detection/'
filenames = [f for f in os.listdir(roi_path) if os.path.isfile(roi_path + f)]

print('Number of frames: {}\n'.format(len(filenames)))

for block_dim in [2, 3, 4, 5]:
    print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
    
    hand_feature_vectors_collect = []
    non_skin_feature_vectors_collect = []
    
    '''initial training'''
    filenames = [\
    #'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
    #'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
    'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
    #'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
    'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
    #'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
    'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
    #'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
    'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
    #'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
    'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
    #'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
    'STI WA May09 B-MVI_0006_trans__600_700_0496.data',
    ]
    
    
#    '''second training where non-skin patches were misclassified as skin by
#    initial training knn models'''
#    filenames = [\
#    'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
#    'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
#    #'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
#    'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
#    #'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
#    'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
#    #'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
#    'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
#    #'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
#    'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
#    #'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
#    'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
#    #'STI WA May09 B-MVI_0006_trans__600_700_0496.data',
#    ]
    
    for filename in filenames:
        
        print(filename)
    
        with open(save+filename[0:-5] + \
                  '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
            
            hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
        
        # I assume there are more non-skin than hand, so verify
        assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
    
        # shuffle, then take a subset of non-skin
        np.random.shuffle(non_skin_feature_vectors)
        
        non_skin_feature_vectors_take = \
            non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
        
        # save for further stacking
        hand_feature_vectors_collect.append(hand_feature_vectors)
        non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
        
    feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
    feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
    
    # hand 1 and non-skin 0
    y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                   np.zeros(feature_vectors_non_skin.shape[0])])
    
    # feature vectors
    X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
    
    # shuffle
    indices_shuffled = np.arange(0, y.shape[0])
    np.random.shuffle(indices_shuffled)
    
    y = y[indices_shuffled]
    X = X[indices_shuffled, :]
    print('X.shape', X.shape)
    
    
    clf_log_reg = sklm.LogisticRegression()
    
    cv_predict = skm.cross_val_predict(clf_log_reg, X, y=y, cv=10, verbose=1)
    
    print('cv_predict.shape', cv_predict.shape)
    
    
    print('\nconfusion matrix:\n', skmet.confusion_matrix(y, cv_predict))
    print('precision:', skmet.precision_score(y, cv_predict))
    print('recall:', skmet.recall_score(y, cv_predict))
    print('F1:', skmet.f1_score(y, cv_predict))
    print('ROC AUC:', skmet.roc_auc_score(y, cv_predict))