# -*- coding: utf-8 -*-
"""
Created on Sun Jan 21 10:24:04 2018

@author: Callie Darsey

Explore the frame resolution, frame skiping, and optical flow parameters for
video by changing the parameters at the top of the script.
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import os

# video path
video_path = './MVI_0053_trans__50_72.mp4'

# Setting to play with

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = 1

'''Resolution rescaling factor'''
scaleFactor = 0.67

'''
normalization of optical flow by resolution scaling factor and 
number of frames advanced
'''
normalize_scale = False
normalize_frame_advance = False

''' wait to calculate optical flow and display '''
frame_wait = 0

'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''

pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0


'''
write out a video
'''
video_write = False

# other settings
displayVideo = True
threshold = 0.0
      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2. CAP_PROP_FPS )
print('number of frames:', num_frames, ', frames per second:', fps)



# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

# prep window
title = 'display window'
#cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.namedWindow(title, cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, 0, -20)
#cv2.moveWindow(title, -468, -1111)

# loop through the frames and display

# announce
print("Press Esc on the image to exit")

e1 = cv2.getTickCount()

count = 0


# for when write_video is true
if video_write == True:
    
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    root, ext = os.path.splitext(video_path)
    video_path_less_frames = root + '__skipped_' + str(framesToAdvance - 1) + ext
    out = cv2.VideoWriter(video_path_less_frames, fourcc,
                          int(np.ceil(fps / framesToAdvance)), 
                          tuple(np.flip(frame1_full.shape[0:2], 0)))
    
    if out.isOpened():
        out.write(frame1_full)
    else:
        print('\nout not open\n')

while(cap.isOpened()):
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            break
     
    if success_flag:
    # process upon true read frame success_flag
    
        if video_write:
    
            if out.isOpened():
                out.write(frame2_full)
            else:
                print('\nout not open\n')

        #initialize frame2
        frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
        frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
        
        
        if count > frame_wait:
            # calculate the flow between frame1 and frame2    
            flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                flow=None,
                                                pyr_scale=pyr_scale, levels=levels,
                                                winsize=winsize,
                                                iterations=iterations,
                                                poly_n=poly_n,
                                                poly_sigma=poly_sigma, flags=flags)
            
            mag = np.sqrt(flow[...,0]**2 + flow[..., 1]**2)
            
            bp1 = plt.boxplot(np.reshape(mag, -1))
            plt.show()
            
            mask1 = mag > bp1['whiskers'][1].get_data()[1][1]
            
            bp2 = plt.boxplot(mag[mask1])
            plt.show()
            
            mask = mag > bp2['medians'][0].get_data()[1][0]
            
            
            mask = np.ones(frame1_gray.shape[0:2])
            
        if displayVideo and count > frame_wait:

            if normalize_scale and normalize_frame_advance:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow * (1 / scaleFactor) / framesToAdvance,
                                       mask, step=4)
            elif normalize_scale:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow * (1 / scaleFactor),
                                       mask, step=4)
            elif normalize_frame_advance:                
                arrows = fns.draw_flow(frame1_gray,
                                       flow / framesToAdvance,
                                       mask, step=4)
            else:                
                arrows = fns.draw_flow(frame1_gray, flow,
                                       mask, step=4)
                
            # draw the text
            cv2.putText(arrows, str(count-framesToAdvance),
                        (arrows.shape[1] - 100, 25),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)            
            
            cv2.imshow(title, arrows)
            #cv2.moveWindow(title, 0, 0)

        # update frame1
        frame1_gray = frame2_gray
                
        # Esc key press will escape
        if count-framesToAdvance == 408:
            k = cv2.waitKey(0) & 0xff
        else:
            k = cv2.waitKey(1) & 0xff
        if k == 27:

            # release capture
            cap.release()

            # close all the OpenCV windows
            cv2.destroyAllWindows()
            
            break
    else:

        # release capture
        cap.release()

        # close all the OpenCV windows
        cv2.destroyAllWindows()

if video_write == True:
    
    out.release()

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)
