# -*- coding: utf-8 -*-
"""
Created on Sun Feb 25 23:44:29 2018

@author: Callie Darsey

Building feature collection for moving hand detection.
"""

import pickle
import cv2
import component_exploration_02 as label_fns
import overlay_01 as overlay
import component_exploration_04 as features
import os


'''
-------------------------------------------------------------------------------
Define the .data files holding resized color frame, flow, and exemplars.
-------------------------------------------------------------------------------
'''

# build up the paths (separated into the directory path and filename)
directory_paths = []
filenames = []

# look for filenames beginning with 
tag_to_look_for = 'frame_flow_exemplars__'
tag_len = len(tag_to_look_for)

# folder to start from
start_folder = os.path.join('.', 'video_copies_clips_track')

# loop through folders and append
for wave_folder in os.listdir(start_folder):

    dir_path_temp = os.path.join(start_folder, wave_folder)
    
    if os.path.isdir(dir_path_temp) == True:
    
        for first_identifying_folder in os.listdir(dir_path_temp):
            
            dir_path_temp = os.path.join(start_folder, wave_folder,
                                         first_identifying_folder)
            
            if os.path.isdir(dir_path_temp) == True:
            
                for second_identifying_folder in os.listdir(dir_path_temp):
                    
                    dir_path_temp = os.path.join(start_folder, wave_folder,
                                                 first_identifying_folder,
                                                 second_identifying_folder)
                    
                    if os.path.isdir(dir_path_temp) == True:
                                    
                        for filename in os.listdir(dir_path_temp):

                            path = os.path.join(dir_path_temp, filename)                            
                            
                            if os.path.isfile(path) == True:
                                
                                if filename[0:tag_len] == tag_to_look_for:
                                    
                                    directory_paths.append(dir_path_temp)
                                    filenames.append(filename)

'''
-------------------------------------------------------------------------------
Loop through the pickled frames presented
-------------------------------------------------------------------------------
'''

escape = False
display = False
save = True

for dir_path, pickled_data_filename in zip(directory_paths, filenames):
    
    if escape == True:
        
        break

    # get the frame, flow, and exemplar info
    path = os.path.join(dir_path, pickled_data_filename)
    print(path)
      
    with open(path, 'rb') as f:
        frame, flow, boxes = pickle.load(f)
        
    # get gray frame
    frame_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        
    # get the connected components in the frame the three ways
    ways = label_fns.get_cc_info(frame, flow, boxes)
    
    # title for displaying the ways
    titles = ['WAY #1: skin, motion, distance transform CCs',
              'WAY #2: break WAY #1 with watershed lines',
              'WAY #3: Use watershed components that touch WAY #1 components']
    
    # get and pickle the targets and features of the components of the 3 ways
    for way, tag in zip(ways, ['way_1_cc_', 'way_2_cc_', 'way_3_cc_']):
        
        if escape == True:
            
            break
        
        # break up the way info
        cc_info, keep_labels, toss_labels = way
        
        # add the features vector to the cc_info structure
        cc_info = list(cc_info)        
        cc_info = features.get_features(cc_info, frame, flow)
        
        # pickle
        out_pickle_data_filename = tag + 'features_and_targets_from_' + \
                                   pickled_data_filename
                                   
        with open(os.path.join(dir_path, out_pickle_data_filename), 'wb') as f:
            pickle.dump((frame, flow, boxes, cc_info), f)
            
        # display if asked for
        if display == True:
            
            overlay_cc = overlay.overlay(frame_gray, cc_info, keep_labels,
                                         (0, 255, 0), alpha=0.5)

            overlay_cc = overlay.overlay(overlay_cc, cc_info, toss_labels,
                                         (0, 0, 255), alpha=0.5)
            
            cv2.namedWindow(tag, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
            cv2.moveWindow(tag, 0, -50) 
            cv2.imshow(tag, overlay_cc)
            
            k = cv2.waitKey(0)
            
            # escape
            if k == 27:
                
                escape = True
        
        
        
cv2.destroyAllWindows()        
