# -*- coding: utf-8 -*-
"""
Created on Sun Apr 15 11:38:24 2018

@author: Callie Darsey

Review how the knn patch skin detector works
"""

import cv2
import numpy as np
import pickle
import os
import fns_investigating_optical_flow_01 as fns
from matplotlib import pyplot as plt
import get_lessened_features_and_save_steps_03 as get
import overlay_01 as overlay


e1 = cv2.getTickCount()

# video path
common_path = './video_copies_clips_track/copies_and_clips/'
specific_path = 'STI Polk Apr20 C/MVI_0053_trans/'
video_name = 'MVI_0053_trans__50_72.mp4'


directory = common_path + specific_path

dir_list = os.listdir(directory)

collect_listings = []

for listing in dir_list:
    
    if 'flow_' + video_name[0:-4] in listing and listing[-5:] == '.data':
        
        collect_listings.append(listing)

collect_listings.sort()

# make the review folder
review = common_path + specific_path + "review_patch_skin_detector/"
if not os.path.isdir(review):
    os.mkdir(review)

# load the skin patch classifier
block_dim = 2
k = 5

path = './video_copies_clips_track/ROI_masks_patch_detection/'
save = path + "block_feature_vectors/"

with open(save + \
          'knn_{:03}_block_dim_{:02d}.data'.format(k, block_dim), 'rb') as f:
    
    clf_knn = pickle.load(f)

# begin counting total time    
e1 = cv2.getTickCount()

# count number of frames processed
count = 0

for listing in collect_listings:
    
    # frame, pre-proccessed number
    count = count + 1
    
    # begin counting time per loop
    eib = cv2.getTickCount()
    
    nframe1 = listing[-14:-10]
    print(nframe1)
    
    # pickle result
    pickle_path = common_path + specific_path + listing
        
    with open(pickle_path, 'rb') as f:
        frame1, frame2, flow, cc_way_1_prev, cc_way_2_prev, cc_way_3_prev = \
            pickle.load(f)
 
    cv2.imwrite(review + 'bgr_' + nframe1 + '.png', frame1)     
       
    intermediate_tag, intermediate_arr = \
        get.get_components_and_intermediate_steps(frame1, flow, 7.0,
                                                  clf_knn, block_dim)
        
    filename_skin_detection, \
    filename_flow_mask, \
    filename_flow_mag_thresh, \
    filename_skin_and_flow_mag_thresh, \
    filename_DT_skin_and_flow_mag_thresh, \
    filename_DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh, \
    filename_way_1_components_mask, \
    filename_DT_from_way_1_components_mask, \
    filename_less_small_connectors, \
    filename_per_cc_top_percent_mask, \
    filename_local_max_mask, \
    filename_markers, \
    filename_watershed_lines, \
    filename_way_2_components_mask, \
    filename_watershed_components, \
    filename_way_3_components_mask =  intermediate_tag
        
    skin_detection, \
    flow_mask, \
    flow_mag_thresh, \
    skin_and_flow_mag_thresh, \
    DT_skin_and_flow_mag_thresh, \
    DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh, \
    way_1_components_mask, \
    DT_from_way_1_components_mask, \
    less_small_connectors, \
    per_cc_top_percent_mask, \
    local_max_mask, \
    markers, \
    watershed_lines, \
    way_2_components_mask, \
    watershed_components, \
    way_3_components_mask = intermediate_arr
    
    print('flow magnitude threshold:', flow_mag_thresh)
    
    cv2.imwrite(review + filename_skin_detection + '_' + nframe1 + '.png',
                skin_detection)
    
    tmp = np.copy(frame1)
    fns.draw_flow(tmp, flow, np.ones(frame1.shape[0:2]), step=4)
    
    cv2.imwrite(review + filename_flow_mask + '_arrows_' + nframe1 + \
                '.png', tmp)
    
    tmp = np.copy(frame1)
    fns.draw_flow(tmp, flow, flow_mask, step=4)
    
    cv2.imwrite(review + filename_flow_mask + '_thresh_arrows_' + \
                nframe1 + '.png', tmp)        

    connectivity = 4
    cc_flow_mask = cv2.connectedComponentsWithStats(flow_mask,
                                                    connectivity,
                                                    cv2.CV_32S)
    
    thr_flow_overlay = overlay.overlay(frame1, cc_flow_mask,
                                       np.unique(cc_flow_mask[1])[1:],
                                       (255, 0, 0), alpha=0.5)
    
    cv2.imwrite(review + filename_flow_mask + '_overlay_' + nframe1 + \
                '.png', thr_flow_overlay)

    cv2.imwrite(review + filename_skin_and_flow_mag_thresh + '_' + \
                nframe1 +'.png', skin_and_flow_mag_thresh)
    
    cv2.imwrite(review + filename_DT_skin_and_flow_mag_thresh + '_' + \
                nframe1 +'.png',
                ((2**16 - 1) * DT_skin_and_flow_mag_thresh / \
                np.max(DT_skin_and_flow_mag_thresh)).astype(np.uint16))
    
    cv2.imwrite(review + \
        filename_DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh + '_' + \
        nframe1 +'.png',
        ((2**16 - 1) * DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh / \
        np.max(DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh)).astype(\
              np.uint16))
    
    cc_way_1 = cv2.connectedComponentsWithStats(way_1_components_mask,
                                                connectivity,
                                                cv2.CV_32S)
    
    way_1_overlay = overlay.overlay(frame1, cc_way_1,
                                    np.unique(cc_way_1[1])[1:],
                                    (255, 157, 0), alpha=0.5)
    
    cv2.imwrite(review + filename_way_1_components_mask + '_' + \
                nframe1 +'.png', way_1_overlay)        
    
    cv2.imwrite(review + filename_DT_from_way_1_components_mask + '_' + \
                nframe1 +'.png',
                ((2**16 - 1) * DT_from_way_1_components_mask / \
                np.max(DT_from_way_1_components_mask)).astype(np.uint16))        

    cv2.imwrite(review + filename_less_small_connectors + '_' + \
                nframe1 +'.png',
                ((2**16 - 1) * less_small_connectors / \
                np.max(less_small_connectors)).astype(np.uint16))    
    
    cv2.imwrite(review + filename_per_cc_top_percent_mask + '_' + \
        nframe1 +'.png', per_cc_top_percent_mask)    

    cv2.imwrite(review + filename_local_max_mask + '_' + \
        nframe1 +'.png', local_max_mask)  

    cv2.imwrite(review + filename_markers + '_' + \
        nframe1 +'.png', ((markers.astype(np.float64) + 1.0) / \
                          np.max(markers.astype(np.float64) + 1.0) * \
                          (2**16 - 1)).astype(np.uint16))

    cv2.imwrite(review + filename_watershed_lines + '_' + \
                nframe1 +'.png',
                ((watershed_lines.astype(np.float64) + 1.0) / \
                 np.max(watershed_lines.astype(np.float64) + 1.0) * \
                 (2**16 - 1)).astype(np.uint16))
    
    cc_way_2 = cv2.connectedComponentsWithStats(way_2_components_mask,
                                                connectivity,
                                                cv2.CV_32S)
    
    way_2_overlay = overlay.overlay(frame1, cc_way_2,
                                    np.unique(cc_way_2[1])[1:],
                                    (255, 242, 0), alpha=0.5)
    
    cv2.imwrite(review + filename_way_2_components_mask + '_' + \
                nframe1 +'.png', way_2_overlay) 
    
    cv2.imwrite(review + filename_watershed_components + '_' + \
                nframe1 +'.png',
                ((watershed_components.astype(np.float64) + 1.0) / \
                 np.max(watershed_components.astype(np.float64) + 1.0) * \
                 (2**16 - 1)).astype(np.uint16))

    cc_way_3 = cv2.connectedComponentsWithStats(way_3_components_mask,
                                                connectivity,
                                                cv2.CV_32S)
    
    way_3_overlay = overlay.overlay(frame1, cc_way_3,
                                    np.unique(cc_way_3[1])[1:],
                                    (255, 187, 0), alpha=0.5)
    
    cv2.imwrite(review + filename_way_3_components_mask + '_' + \
                nframe1 +'.png', way_3_overlay)

    # save the components        
    with open(pickle_path, 'wb') as f:
        pickle.dump((frame1, frame2, flow, \
                     cc_way_1, cc_way_2, cc_way_3), f)
        
    eie = cv2.getTickCount()
    time = (eie - eib)/ cv2.getTickFrequency()
    print("Seconds to process frame and pre-processed flow:", time)   

print('\nProccessed {} frames and pre-processed flow:', count)        
e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Total execution time in seconds = ", time)