# -*- coding: utf-8 -*-
"""
Created on Sun May  6 13:56:08 2018

@author: Callie Darsey

make the arrows video on wheeler
"""

import cv2
import pickle
import numpy as np
import os

def draw_flow(frame, frame_flow, mask, step=1):
    """ Plot optical flow at sample points spaced step pixels apart. """
    
    # sample the points at which the vectors will be drawn
    
    # get image dimensions
    h,w = frame.shape[:2]
    
    # get row and column indices for all points in two 1D numpy arrays
    y,x = np.mgrid[np.floor(step/2):h:step, \
                   np.floor(step/2):w:step].reshape(2,-1)
    
    # get the flow components at each point
    flow_x, flow_y = frame_flow[y.astype(int), x.astype(int)].T
        
    # create line endpoints
    lines = np.vstack([x, y, x + flow_x, y + flow_y]).T.reshape(-1,2,2)
    lines = np.int32(lines)
    
    if len(frame.shape) == 2:
        frame_gray_3D = np.stack([frame, frame, frame], axis=2)
    else:
        frame_gray_3D = frame

    for (x1,y1),(x2,y2) in lines:
 
        # check if the sample point is within the regions kept after thresholding
        cond = (mask[y1, x1] == True or mask[y1, x1] != 0)
        if (cond):
            cv2.line(frame_gray_3D,(x1,y1),(x2,y2),(0,255,0),1)
            #cv2.circle(frame_gray_3D,(x1,y1),1,(0,255,0), -1)
    
    return frame_gray_3D

def function():
    
    e1 = cv2.getTickCount()
    
    # video path
    common_path = './'
    flow_folder = 'flow/'
    
    pickled_flow_paths = \
        [common_path + flow_folder + data_file \
         for data_file in os.listdir(common_path + flow_folder)]
        
    pickled_flow_paths.sort()
    
    name = pickled_flow_paths[0][7:-15]
    
    with open(pickled_flow_paths[0], 'rb') as f:
        flow = pickle.load(f)
        
    '''
    Write sampled arrows display to a video.
    '''
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    
    out_arrows = cv2.VideoWriter(common_path +  name + '.arrows.mp4',
                                 fourcc, 15, flow[:, :, 0].shape[-1::-1])
        
    '''
    Read from sampled video
    '''
    sampled = common_path + pickled_flow_paths[0][12:-15] + '.sampled.mp4'
    
    cap = cv2.VideoCapture(sampled)
    
    num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    fps = cap.get(cv2. CAP_PROP_FPS )
    
    print('number of frames:', num_frames)
    print('frames per second:', fps)


    # loop
    count = -1

    while(cap.isOpened()):
        
        eb = cv2.getTickCount()
        
        count = count + 1
        
        success_flag, frame = cap.read()

        if success_flag:
        # process upon true read frame success_flag
    
            with open(pickled_flow_paths[count], 'rb') as f:
                flow = pickle.load(f)

            arrows = draw_flow(frame, flow, np.ones(flow[:, :, 0].shape),
                               step=4)
            
            # save frame
            out_arrows.write(arrows)
            
            
            ee = cv2.getTickCount()
            print('per frame {:.2f} seconds'.format(\
                  (ee - eb)/ cv2.getTickFrequency()))
    
        else:
    
            # release capture
            cap.release()
            out_arrows.release()
    
    e2 = cv2.getTickCount()
    time = (e2 - e1)/ cv2.getTickFrequency()
    print("Execution time in seconds = ", time)
