# -*- coding: utf-8 -*-
"""
Created on Thu Apr 12 15:28:44 2018

@author: Callie Darsey

Abby's skin detector

skin_detector_Abby_01
"""

import cv2
import numpy as np
import os
import component_functions_01 as comp_fns


def prep_window(title):

    move_window = [0, -30]
    #cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.namedWindow(title, cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1])
    
def DetectSkinRegions_02(frame):
    '''
    Return the detected skin regions mask that uses the values Abby's
    skin detector has but does not have the cleanup done
    
    '''

    f = np.copy(frame)
    min_YCrCb  = np.array([ 75, 138, 111], np.uint8)
    max_YCrCb  = np.array([172, 155, 124], np.uint8)
    min_YCrCb1 = np.array([ 56, 142, 110], np.uint8)

    # Convert image to YCrCb
    image_YCrCb = cv2.cvtColor(f,cv2.COLOR_BGR2YCrCb)
    
    # Blur image_YCrCb
    imgYCC = cv2.GaussianBlur(image_YCrCb, (11, 11), 0)
    
    # Find region with skin tone in YCrCb image
    skin_region  = cv2.inRange(imgYCC, min_YCrCb,  max_YCrCb)
    skin_region2 = cv2.inRange(imgYCC, min_YCrCb1, max_YCrCb)
    
    # practically anding skin_region and skin_region2   
    skin_region = cv2.bitwise_and(skin_region, skin_region, mask=skin_region2)
    
    # fill holes
    comp_fns.fill_holes()

    return skin_region
    
def DetectSkinRegions(frame):
    '''
    Return the detected skin regions mask that uses the values Abby's
    skin detector has but does not have the cleanup done
    
    '''

    f = np.copy(frame)
    min_YCrCb  = np.array([ 75, 138, 111], np.uint8)
    max_YCrCb  = np.array([172, 155, 124], np.uint8)
    min_YCrCb1 = np.array([ 56, 142, 110], np.uint8)

    # Convert image to YCrCb
    image_YCrCb = cv2.cvtColor(f,cv2.COLOR_BGR2YCrCb)
    
    # Blur image_YCrCb
    imgYCC = cv2.GaussianBlur(image_YCrCb, (11, 11), 0)
    
    # Find region with skin tone in YCrCb image
    skin_region  = cv2.inRange(imgYCC, min_YCrCb,  max_YCrCb)
    skin_region2 = cv2.inRange(imgYCC, min_YCrCb1, max_YCrCb)
    
    # practically anding skin_region and skin_region2   
    skin_region = cv2.bitwise_and(skin_region, skin_region, mask=skin_region2)

    return skin_region



def PersonObject(frame, show=True):
    
    kernel  = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (11,11))
    kernel2 = cv2.getStructuringElement(cv2.MORPH_RECT, (11,11))
    
    f = np.copy(frame)
    min_YCrCb  = np.array([ 75, 138, 111], np.uint8)
    max_YCrCb  = np.array([172, 155, 124], np.uint8)
    min_YCrCb1 = np.array([ 56, 142, 110], np.uint8)

    # Convert image to YCrCb
    image_YCrCb = cv2.cvtColor(f,cv2.COLOR_BGR2YCrCb)
    
    # Blur image_YCrCb
    imgYCC = cv2.GaussianBlur(image_YCrCb, (11, 11), 0)
    
    # Find region with skin tone in YCrCb image
    skin_region  = cv2.inRange(imgYCC, min_YCrCb,  max_YCrCb)
    skin_region2 = cv2.inRange(imgYCC, min_YCrCb1, max_YCrCb)
    
    # practically anding skin_region and skin_region2   
    skin_region = cv2.bitwise_and(skin_region, skin_region, mask=skin_region2)

    # show the skin regions remaining with the mask
    pre_cleanup = cv2.bitwise_and(f, f, mask=skin_region)  
    if show == True:         
        t = 'skin mask before cleaning'
        prep_window(t), cv2.imshow(t, pre_cleanup)
    
    # open    
    skin_region = cv2.morphologyEx(skin_region, cv2.MORPH_OPEN, kernel)
    
    if show==True:
        t = 'open'
        prep_window(t), cv2.imshow(t, skin_region)
    
    # dilate a lot but with a small 3x3 rectangular kernel
    skin_region = cv2.dilate(skin_region, None, iterations=10)
    
    
    
    # close
    skin_region = cv2.morphologyEx(skin_region, cv2.MORPH_CLOSE, kernel)
    
    # Blur the mask (I guess this extends the edges)
    skin_region = cv2.GaussianBlur(skin_region, (29, 29), 0)
    
    # Erode
    skin_region = cv2.erode(skin_region, kernel2, iterations=2)
    
    # open
    skin_region = cv2.morphologyEx(skin_region, cv2.MORPH_OPEN, kernel,
                                   iterations=2)

    
    # show the skin regions remaining with the mask
    res = cv2.bitwise_and(f, f, mask=skin_region)
    if show==True:
        t = 'skin cleaned up skin mask';
        prep_window(t); cv2.imshow(t, res)
        
    return res

if __name__ == '__main__':
    # video path
    common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
    specific_path = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
    video_name = 'MVI_0028_trans__200_208.mp4'
    review = common_path + specific_path + "PNGs/"
    
    filenames = [ fn  for fn in os.listdir(review) \
                      if os.path.isfile(review + '/' + fn)]

    
    for fn in filenames:
    
        frame = cv2.imread(review + fn)

        cv2.putText(frame, fn[-8:-4], (frame.shape[1] - 100, 25),
                    cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
                        
        t = 'original image'; prep_window(t); cv2.imshow(t, frame)
        
            
        skin_detection = PersonObject(frame)
        
        
        k = cv2.waitKey(0)
        
        if k == 27:
            
            cv2.destroyAllWindows()
            
            break
            
            
        cv2.destroyAllWindows()