# -*- coding: utf-8 -*-
"""
Created on Sun Mar  4 17:35:47 2018

@author: Callie Darsey (cj.darsey@gmail.com)

I want to try out a RandomForestClassifier
"""

import pickle
import numpy as np
import sklearn.ensemble as sk_ensemble

files = ['way_1_cc_features.data', 'way_2_cc_features.data',
         'way_3_cc_features.data']

for path in files:
    
    print('\n\n', path, '\n')
    
    # load the pickled features
    with open(path, 'rb') as f:
        keep_features, toss_features = pickle.load(f)
        
    # print the training set size
    n_keep_samples, n_keep_features = keep_features.shape
    n_toss_samples, n_toss_features = toss_features.shape
    
    assert n_keep_features == n_toss_features
    
    n_features = n_keep_features
    
    print('Number of features: {}'.format(n_features))
    print('Number of keep samples available: {}'.format(n_keep_samples))
    print('Number of toss samples available: {}'.format(n_toss_samples))

    out_of_bag_score = []
    
    trials = 1
    for i in range(trials):
        '''
        Force the class with more samples to have just as many samples by randomly
        choosing a subset.
        '''
        if n_keep_samples > n_toss_samples:
            
            np.random.shuffle(keep_features)
            
            X = np.vstack([keep_features[0:n_toss_samples, :], toss_features])
        
            y = np.hstack([np.ones(n_toss_samples, dtype=np.int),
                           np.zeros(n_toss_samples, dtype=np.int)])
    
        elif n_keep_samples < n_toss_samples:
            
            np.random.shuffle(toss_features)
            
            X = np.vstack([keep_features, toss_features[0:n_keep_samples, :]])
        
            y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                           np.zeros(n_keep_samples, dtype=np.int)])
        
        else:
        
            X = np.vstack([keep_features, toss_features])
        
            y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                           np.zeros(n_toss_samples, dtype=np.int)])
    
        # shuffle the training set and targets
        n_samples = X.shape[0]
        
        shuffled_indices = np.arange(n_samples)
        np.random.shuffle(shuffled_indices)
        
        X = X[shuffled_indices, :]
        y = y[shuffled_indices]
        
        # make the classifier
        n_estimators=200
        print(n_estimators, 'trees in the forest')
        rf_clf = sk_ensemble.RandomForestClassifier(n_estimators=n_estimators,
                                                    oob_score=True)
        
        # train
        rf_clf.fit(X, y)
        
        # pickle the classifier 'way_1_cc_features.data'
        with open(path[0:9] + 'RandomForestClassifier' + '.data', 'wb') as f:
            pickle.dump((rf_clf), f)
            
        
        # look at the out-of-bag scores
        print('out-of-bag accuracy score: {:.4f}'.format(rf_clf.oob_score_))
        
        out_of_bag_score.append(rf_clf.oob_score_)
    
    print('For', trials, 'trials: ')
    print('   min out-of-bag accuracy score:', np.min(out_of_bag_score))
    print('  mean out-of-bag accuracy score:', np.mean(out_of_bag_score))
    print('median out-of-bag accuracy score:', np.median(out_of_bag_score))
    print('   max out-of-bag accuracy score:', np.max(out_of_bag_score))
                
    