# -*- coding: utf-8 -*-
"""
Created on Sat Mar  3 13:33:59 2018

@author: Callie Darsey

I want to run the t-test over each keep / toss feature.

References:
    
https://towardsdatascience.com/inferential-statistics-series-t-test-using-numpy-2718f8f9bf2f

Per Heidi Darsey (undergrad minoring in statistics), in section 6., cross out
the " and we accept the alternate hypothesis which says that the height of men
and women are statistically different" as incorrect. For the p-value
description, "A p-value is the probability that the results from your sample
data occurred by chance", add the phrase 'given the null hypothesis' to the
end. Heidi described the p-value as "The p-value is the probability that you
observe data at least as extreme (towards the tails) as your data , under the
assumption that the means are equal."

https://docs.scipy.org/doc/scipy-0.15.1/reference/generated/scipy.stats.ttest_ind.html

https://en.wikipedia.org/wiki/Welch%27s_t-test

https://en.wikipedia.org/wiki/Variance

https://en.wikipedia.org/wiki/One-_and_two-tailed_tests

https://en.wikipedia.org/wiki/Student%27s_t-distribution

https://en.wikipedia.org/wiki/P-value

http://blog.minitab.com/blog/statistics-and-quality-data-analysis/what-are-t-values-and-p-values-in-statistics

https://onlinecourses.science.psu.edu/statprogram/review_of_basic_statistics

http://mathworld.wolfram.com/BonferroniCorrection.html
"""

import os
import pickle
import numpy as np
import component_exploration_04 as fts
from scipy import stats

'''
-------------------------------------------------------------------------------
Define the .data files holding the connected component information from
cv2.connectedComponentsWithStats() with appended elements for component target
and component features. The .data files hold the resized color frame, flow,
and exemplars as well.
-------------------------------------------------------------------------------
'''



for tag_to_look_for in \
    ['way_1_cc_features_and_targets_from_frame_flow_exemplars__',
     'way_2_cc_features_and_targets_from_frame_flow_exemplars__',
     'way_3_cc_features_and_targets_from_frame_flow_exemplars__']:
        
    # build up the paths (separated into the directory path and filename)
    directory_paths = []
    filenames = []
        
    print('\n****************************************************************')
    print(tag_to_look_for)
    print('****************************************************************\n')
    
    # look for filenames beginning with 
    tag_len = len(tag_to_look_for)
    
    # folder to start from
    start_folder = os.path.join('.', 'video_copies_clips_track')
    
    # loop through folders and append
    for wave_folder in os.listdir(start_folder):
    
        dir_path_temp = os.path.join(start_folder, wave_folder)
        
        if os.path.isdir(dir_path_temp) == True:
        
            for first_identifying_folder in os.listdir(dir_path_temp):
                
                dir_path_temp = os.path.join(start_folder, wave_folder,
                                             first_identifying_folder)
                
                if os.path.isdir(dir_path_temp) == True and \
                    dir_path_temp[-3:] != 'XXX':
                
                    for second_identifying_folder in os.listdir(dir_path_temp):
                        
                        dir_path_temp = os.path.join(start_folder, wave_folder,
                                                     first_identifying_folder,
                                                     second_identifying_folder)
                        
                        if os.path.isdir(dir_path_temp) == True:
                                        
                            for filename in os.listdir(dir_path_temp):
    
                                path = os.path.join(dir_path_temp, filename)                            
                                
                                if os.path.isfile(path) == True:
                                    
                                    if filename[0:tag_len] == tag_to_look_for:
                                        
                                        directory_paths.append(dir_path_temp)
                                        filenames.append(filename)
                                        
    '''
    Count the examples
    '''
    
    counting = {}
    number_of_keep_CCs_index = 0
    number_of_toss_CCs_index = 1
    
    keep_features = []
    toss_features = []
    
    for dir_path, filename in zip(directory_paths, filenames):
        
        path = os.path.join(dir_path, filename)
        
        with open(path, 'rb') as f:
            frame, flow, boxes, cc_info = pickle.load(f)
            
        features_stacked = np.vstack(cc_info[-1])
        targets_stacked = np.array(cc_info[-2]).reshape((-1, 1))
    
            
        keep_ind = np.where(targets_stacked == 1)[0]
        toss_ind = np.where(targets_stacked == 0)[0]
        
        if len(keep_features) == 0:
            keep_features = features_stacked[keep_ind, :]
        else:
            keep_features = np.vstack([keep_features,
                                       features_stacked[keep_ind, :]])
        
        if len(toss_features) == 0:
            toss_features = features_stacked[toss_ind, :]
        else:
            toss_features = np.vstack([toss_features,
                                       features_stacked[toss_ind, :]])
        
        ind = filename.rfind('__')
        identifying_part_of_name = filename[tag_len:ind]
        preceding, second_identifying_folder = os.path.split(dir_path)
        preceding, first_identifying_folder = os.path.split(preceding)
        
        identifier = os.path.join(first_identifying_folder,
                                  identifying_part_of_name)
        
        if  identifier not in counting.keys():
                
            counting[identifier] = [[], []]
            
        counting[identifier][number_of_keep_CCs_index].append(keep_ind.shape[0])
        counting[identifier][number_of_toss_CCs_index].append(toss_ind.shape[0])
        
    # pickle the features
    pickle_path = tag_to_look_for[0:17] + '.data'
    with open(pickle_path, 'wb') as f:
        pickle.dump((keep_features, toss_features), f)
    
    
    # print number of keep versus toss component examples
    print('\nNumber of Components Given Keep Targets:', keep_features.shape[0])
    print('\nNumber of Components Given Toss Targets:', toss_features.shape[0])
    
    # print number of clips
    print('\nNumber of clips:', len(counting.keys()))
    
    frames_per_clip = []
    
    keep_per_frame = []
    toss_per_frame = []
    
    keep_per_clip = []
    toss_per_clip = []
    
    for clip in counting.keys():
        
        # frames per clip
        frames_per_clip.append(len(counting[clip][number_of_keep_CCs_index]))
        
        # keep or toss per frame
        if len(keep_per_frame) == 0:
            keep_per_frame = np.array(counting[clip][number_of_keep_CCs_index])
        else:
            keep_per_frame = np.hstack([keep_per_frame,
                np.array(counting[clip][number_of_keep_CCs_index])])
            
        if len(toss_per_frame) == 0:
            toss_per_frame = np.array(counting[clip][number_of_toss_CCs_index])
        else:
            toss_per_frame = np.hstack([toss_per_frame,
                np.array(counting[clip][number_of_toss_CCs_index])])
        
        # keep or toss per clip
        if len(keep_per_clip) == 0:
            keep_per_clip = \
                np.array([np.sum(counting[clip][number_of_keep_CCs_index])])
        else:
            keep_per_clip = np.hstack([keep_per_clip,
                np.array([np.sum(counting[clip][number_of_keep_CCs_index])])])
            
        if len(toss_per_clip) == 0:
            toss_per_clip = \
                np.array([np.sum(counting[clip][number_of_toss_CCs_index])])
        else:
            toss_per_clip = np.hstack([toss_per_clip,
                np.array([np.sum(counting[clip][number_of_toss_CCs_index])])])
    
    print('\nFrames taken per clip:')
    print('\tMinimum', np.min(np.array(frames_per_clip)))
    print('\tMean', np.mean(np.array(frames_per_clip)))
    print('\tMedian', np.median(np.array(frames_per_clip)))
    print('\tMaximum', np.max(np.array(frames_per_clip)))
    
    print('\nkeep CCs per frame:\t\t\tToss CCs per frame:')
    print('\tMinimum {:d}\t\t\t\tMinimum {:d}'.format(np.min(keep_per_frame),
                                                      np.min(toss_per_frame)))
    print('\tMean {:.1f}\t\t\t\tMean {:.1f}'.format(np.mean(keep_per_frame),
                                                      np.mean(toss_per_frame)))
    print('\tMedian {:.1f}\t\t\t\tMedian {:.1f}'.format(np.median(keep_per_frame),
                                                      np.median(toss_per_frame)))
    print('\tMaximum {:d}\t\t\t\tMaximum {:d}'.format(np.max(keep_per_frame),
                                                      np.max(toss_per_frame)))
    
    print('\nkeep CCs per clip:\t\t\tToss CCs per clip:')
    print('\tMinimum {:d}\t\t\t\tMinimum {:d}'.format(np.min(keep_per_clip),
                                                      np.min(toss_per_clip)))
    print('\tMean {:.1f}\t\t\t\tMean {:.1f}'.format(np.mean(keep_per_clip),
                                                      np.mean(toss_per_clip)))
    print('\tMedian {:.1f}\t\t\t\tMedian {:.1f}'.format(np.median(keep_per_clip),
                                                      np.median(toss_per_clip)))
    print('\tMaximum {:d}\t\t\t\tMaximum {:d}'.format(np.max(keep_per_clip),
                                                      np.max(toss_per_clip)))
    
    print('')
    # Display t-statistic, p-value
    start_end_pairs_256_1 = [[fts.FEATURE_B_HIST_LEFT,
                              fts.FEATURE_B_HIST_RIGHT + 1],
                             [fts.FEATURE_G_HIST_LEFT,
                              fts.FEATURE_G_HIST_RIGHT + 1],
                             [fts.FEATURE_R_HIST_LEFT,
                              fts.FEATURE_R_HIST_RIGHT + 1]]
        
    start_end_pair_180_1 = [[fts.FEATURE_H_HIST_LEFT,
                             fts.FEATURE_H_HIST_RIGHT + 1]]
    
    start_end_pair_256_2 = [[fts.FEATURE_S_HIST_LEFT,
                             fts.FEATURE_S_HIST_RIGHT + 1],
                            [fts.FEATURE_V_HIST_LEFT,
                             fts.FEATURE_V_HIST_RIGHT + 1],
                            [fts.FEATURE_Y_HIST_LEFT,
                             fts.FEATURE_Y_HIST_RIGHT + 1],
                            [fts.FEATURE_Cr_HIST_LEFT,
                             fts.FEATURE_Cr_HIST_RIGHT + 1],
                            [fts.FEATURE_Cb_HIST_LEFT,
                             fts.FEATURE_Cb_HIST_RIGHT + 1],
                             
                            [fts.FEATURE_B_PDF_LEFT,
                             fts.FEATURE_B_PDF_RIGHT + 1],
                            [fts.FEATURE_G_PDF_LEFT,
                             fts.FEATURE_G_PDF_RIGHT + 1],
                            [fts.FEATURE_R_PDF_LEFT,
                             fts.FEATURE_R_PDF_RIGHT + 1]]
    
    start_end_pair_180_2 = [[fts.FEATURE_H_PDF_LEFT,
                             fts.FEATURE_H_PDF_RIGHT + 1]]
                                              
    start_end_pairs_256_3 = [[fts.FEATURE_S_PDF_LEFT,
                              fts.FEATURE_S_PDF_RIGHT + 1],
                             [fts.FEATURE_V_PDF_LEFT,
                              fts.FEATURE_V_PDF_RIGHT + 1],
                             [fts.FEATURE_Y_PDF_LEFT,
                              fts.FEATURE_Y_PDF_RIGHT + 1],
                             [fts.FEATURE_Cr_PDF_LEFT,
                              fts.FEATURE_Cr_PDF_RIGHT + 1],
                             [fts.FEATURE_Cb_PDF_LEFT,
                              fts.FEATURE_Cb_PDF_RIGHT + 1]]
               
    start_end_pairs_64 = [[fts.FEATURE_DFT_MAG_SPEC_HIST_LEFT,
                           fts.FEATURE_DFT_MAG_SPEC_HIST_RIGHT + 1],
                          [fts.FEATURE_DFT_PHASE_SPEC_HIST_LEFT,
                           fts.FEATURE_DFT_PHASE_SPEC_HIST_RIGHT + 1]]
        
    start_end_pairs_8 = \
              [[fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_LEFT,
                fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_RIGHT],
               [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT,
                fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT + 1],
               [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT,
                fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT + 1]]
                     
    start_end_titles_256_1 = ['Blue Histogram',
                              'Green Histogram',
                              'Red Histogram',
                              'Hue Histogram',
                              'Saturation Histogram']
    
    start_end_titles_180_1 = ['Value Histogram']
    
    start_end_titles_256_2 = ['Luma Histogram',
                              'Red-difference Chroma Component Histogram',
                              'Blue-difference Chroma Component Histogram',
                        
                              'Blue PDF',
                              'Green PDF',
                              'Red PDF']
    
    start_end_titles_180_2 = ['Hue PDF']
    
    start_end_titles_256_3 = ['Saturation PDF',
                              'Value PDF',
                              'Luma PDF',
                              'Red-difference Chroma Component PDF',
                              'Blue-difference Chroma Component PDF']
                        
    start_end_titles_64 = ['DFT Magnitude Spectrum Histogram',
                           'DFT Phase Spectrum Histogram']
                            
    start_end_titles_8 = ['Motion histogram',
                          'Motion histogram normalized by count',
                          'Motion histogram normalized by area']
    
    single_indices = [fts.FEATURE_CANNY_EDGE_TO_AREA_RATIO,
                      fts.FEATURE_AREA,
                      fts.FEATURE_PERIMETER,
                      fts.FEATURE_MIN_AREA_RECT_ORIENT,
                      fts.FEATURE_AREA_TO_MIN_AREA_RECT,
                      fts.FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO]
    
    single_titles = ['Canny edge to area ratio',
                     'Area',
                     'Perimeter',
                     'Orientation of Minimum Area Bounding Box',
                     'Area to Minimum Area Bounding Box Area',
                     'Short to Long Sides Ratio of Minimum Area Bounding Box']
    
    num_of_features = keep_features.shape[1]
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    start_end_pairs = start_end_pairs_256_1 + start_end_pair_180_1 + \
                      start_end_pair_256_2 + start_end_pair_180_2 + \
                      start_end_pairs_256_3 + start_end_pairs_64 + \
                      start_end_pairs_8
    
    start_end_titles = start_end_titles_256_1 + start_end_titles_180_1 + \
                       start_end_titles_256_2 + start_end_titles_180_2 + \
                       start_end_titles_256_3 + start_end_titles_64 + \
                       start_end_titles_8

    
    single_indices = [fts.FEATURE_CANNY_EDGE_TO_AREA_RATIO,
                      fts.FEATURE_AREA,
                      fts.FEATURE_PERIMETER,
                      fts.FEATURE_MIN_AREA_RECT_ORIENT,
                      fts.FEATURE_AREA_TO_MIN_AREA_RECT,
                      fts.FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO]
    
    single_titles = ['Canny edge to area ratio',
                     'Area',
                     'Perimeter',
                     'Orientation of Minimum Area Bounding Box',
                     'Area to Minimum Area Bounding Box Area',
                     'Short to Long Sides Ratio of Minimum Area Bounding Box']    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    for title, start_end in zip(start_end_titles, start_end_pairs):
        print('\n')
        print(title, '\n')
        
        s = ''
        t = ''
        p = ''
        for i in range(start_end[0], start_end[1]):

            if not ((i - start_end[0]) % 16):
                
                t_statistic, p_value = stats.ttest_ind(keep_features[:, i],
                                                       toss_features[:, i])
                
                if (i - start_end[0]) != 0:
#                    print(s)
                    print(t)
                    print(p, '\n')
                    
                s = 'color space value:    {:6d}'.format(i - start_end[0])
                t = 't-statistic: {:16.3f}'.format(t_statistic)
                p = 'p-value:     {:16.3f}'.format(p_value * num_of_features)
                
            else:
                s = s + '{:10d}'.format(i - start_end[0])
                t = t + '{:10.3f}'.format(t_statistic)
                p = p + '{:10.3f}'.format(p_value * num_of_features)
        else:
#            print(s)
            print(t)
            print(p)
    
        
    
        
    for title, index in zip(single_titles, single_indices):
    
        print('\n')
        print(title, '\n')

        t_statistic, p_value = stats.ttest_ind(keep_features[:, index],
                                               toss_features[:, index])
        print('t-statistic: {:10.3f}'.format(t_statistic))
        print('p-value:     {:10.3f}'.format(p_value * num_of_features))




