# -*- coding: utf-8 -*-
"""
Created on Thu Feb  1 16:30:11 2018

@author: Callie Darsey (cj.darsey@gmail.com)

Try Canny Edge Detection and Hough Line Tranform and try skin detection
on AOLME images.
"""


import cv2
import numpy as np
import skin_detector_01 as skin_detector

# path and name parts for images
path_common = './video_copies_clips_track/copies_and_clips/'
path_specific = '/STI WA Apr25 A/STI WA Apr25 AMVI_0068_trans/'
video_name = 'STI WA Apr25 AMVI_0068_trans__860_910.mp4'

# frame numbers to look at (frame numbers part of image name)
#numbers = [33]
numbers = [33, 69, 265, 297, 393, 437, 545, 773, 1061, 1197, 1237, 1297,
           1665, 1725]

for number in numbers:
    
    # read the image
    image_path = path_common + path_specific + 'images/' + \
        video_name[0:-4] + '__'+ '{:05d}'.format(number) + '.png'

    img = cv2.imread(image_path)
    
    # get the skin mask
    skin_mask = skin_detector.DetectSkinRegions(img)
    
    # get the canny edge detector
    gray = cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
    edges = cv2.Canny(gray,50,150,apertureSize = 3)
    #kernel = np.ones((3,3), dtype=np.uint8)
    #edges = cv2.dilate(edges,kernel,iterations = 1)
    
    # get the probabilistic hough line detection
    rho_resolution = 1
    theta_resolution = np.pi / 180
    accumulator_threshold = 100
    minLineLength = 100
    maxLineGap = 10
    lines = cv2.HoughLinesP(edges,rho_resolution,theta_resolution,
                            accumulator_threshold,minLineLength,maxLineGap)
    
    img_with_lines = img.copy()
    for x1,y1,x2,y2 in np.reshape(lines, (lines.shape[0], lines.shape[2])):
        
        cv2.line(img_with_lines,(x1,y1),(x2,y2),(0,255,0), 1)
        
   
    # display    
    canny_title = 'canny edges'
    prob_hough_title = 'line detection'
    skin_title = 'skin detection'


    cv2.namedWindow(prob_hough_title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(prob_hough_title, 10, -20)    
    cv2.namedWindow(canny_title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(canny_title, 0, -20)
    cv2.namedWindow(skin_title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(skin_title, 20, -20)

        
    cv2.imshow(skin_title, cv2.bitwise_and(img, img, mask=skin_mask))
    cv2.imshow(prob_hough_title, img_with_lines)
    cv2.imshow(canny_title, edges)

    
    cv2.waitKey(0)
    
    cv2.destroyAllWindows()

