# -*- coding: utf-8 -*-
"""
Created on Sat Feb  3 15:10:15 2018

@author: Callie Darsey (cj.darsey@gmail.com)

I want a script for viewing a video while the frame numbers are displayed in
the frame. I want to be able to pause and go back from the paused frame and 
so that I can write down (in my scratch notebook) which frames I want to 
save as images and have the optical flow for while looking into detecting
moving hands. 
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import os
import overlay_01 as overlay

# video path
paths_specific = ['/STI WA Apr25 A/STI WA Apr25 AMVI_0068_trans/',
                  'STI Polk Mar02 E/00005_trans/',
                  'STI Polk Apr13 E/MVI_0044_trans/',
                  'STI Polk Mar02 B/00006_trans/',
                  'STI WA Mar28 C/MVI_0013_trans/',
                  'STI WA May06 Polk B/MVI_0091_trans/',
                  'STI Polk Feb16 C/00054_trans/',
                  'STI Polk May04 F/MVI_0079_trans/',
                  'STI WA Apr18 D/MVI_0056_trans/',
                  'STI WA Feb28 xtraB/00011_trans/',
                  'STI Polk Apr20 C/MVI_0053_trans/',
                  'STI WA Apr18 B/MVI_0029_trans/',
                  'STI WA Feb28 C/00089_trans/']
video_names = ['STI WA Apr25 AMVI_0068_trans__860_910.mp4',
               '00005_trans__605_628.mp4',
               'MVI_0044_trans__350_365.mp4',
               '00006_trans__567_600.mp4',
               'MVI_0013_trans__110_140.mp4',
               'MVI_0091_trans__60_85.mp4',
               '00054_trans__452_505.mp4',
               'MVI_0079_trans__390_415.mp4',
               'MVI_0056_trans__205_240.mp4',
               '00011_trans__290_335.mp4',
               'MVI_0053_trans__50_72.mp4',
               'MVI_0029_trans__75_125.mp4',
               '00089_trans__110_165.mp4']
frames_target = [[ 260,  748, 1200, 1480, 1648, 1660, 1780],
                 [  78,  164,  190,  202,  206,  322,  406,  442,  508,  554],
                 [  92,  248,  344,  404,  480,  544,  784],
                 [  84,  170,  316,  408,  500,  792,  870],
                 [   4,  272,  316,  620,  704,  940, 1140, 1260],
                 [  40,  212,  396,  480,  620,  792,  880],
                 [ 122,  360,  478,  632,  876,  906,  938, 1144, 1164, 1196],
                 [   0,  144,  688,  964],
                 [  76,  126,  814],
                 [ 134,  190,  202,  770,  778,  908, 1110],
                 [  76,  736,  800,  860, 1132, 1264],
                 [ 172,  324,  472,  596, 1136, 1720, 2000, 2128, 2396, 2576],
                 [  16,   22,  462,  478,  560,  712,  776,  784,  836, 1066, 1238]]
path_common = './video_copies_clips_track/copies_and_clips/'
path_specific = ''
video_name = ''
video_path = path_common + path_specific + video_name
             

# Setting to play with

'''Resolution rescaling factor'''
scaleFactor = 0.7

'''
normalization of optical flow by resolution scaling factor and 
number of frames advanced
'''
normalize_scale = True
normalize_frame_advance = False

''' wait to calculate optical flow and display '''
frame_wait = 0

'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''

pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0


'''
write out a video
'''
video_write = False

'''
write out images
'''
image_write = False

'''
Whether to display the video
'''
displayVideo = True

'''
Whether to calculate optical flow
'''
calc_flow = False

'''
insert frame number into frame
'''
insert_frame_number = True

if insert_frame_number == True:
    
    digits = overlay.get_digits()

      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2.CAP_PROP_FPS )

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = np.int32(np.ceil(fps / 15))

print('number of frames:', num_frames, ', frames per second:', fps,
      'framesToAdvance:', framesToAdvance)



# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

'''
save frames
'''
number_saved = 50
saved_frames = []
ind = -1

# prep window
title = 'display window'
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, 0, -30)
#cv2.moveWindow(title, -468, -1111)
#cv2.moveWindow(title, -234, -900)

# loop through the frames and display

# announce
print("Press Esc on the image to exit")

e1 = cv2.getTickCount()

count = 0


# for when write_video is true
if video_write == True:
    
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    root, ext = os.path.splitext(video_path)
    video_path_less_frames = root + '__skipped_' + str(framesToAdvance - 1) + ext
    out = cv2.VideoWriter(video_path_less_frames, fourcc, 20, 
                          tuple(np.flip(frame1_full.shape[0:2], 0)))
    
    if out.isOpened():
        out.write(frame1_full)
    else:
        print('\nout not open\n')
        
# for when image_write is true
if image_write == True:
    
    image_path = path_common + path_specific + 'images/' + \
        video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
    
    cv2.imwrite(image_path, frame1)

while(cap.isOpened()):
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            break
     
    if success_flag:
    # process upon true read frame success_flag
    
        if video_write:
    
            if out.isOpened():
                out.write(frame2_full)
            else:
                print('\nout not open\n')
                
        #initialize frame2
        frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
        frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)

        if image_write == True:    
            
            image_path = path_common + path_specific + 'images/' + \
            video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
            
            cv2.imwrite(image_path, frame2)
        
        
        if count > frame_wait and calc_flow == True:
            # calculate the flow between frame1 and frame2    
            flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                flow=None,
                                                pyr_scale=pyr_scale, levels=levels,
                                                winsize=winsize,
                                                iterations=iterations,
                                                poly_n=poly_n,
                                                poly_sigma=poly_sigma, flags=flags)
            
            mag = np.sqrt(flow[...,0]**2 + flow[..., 1]**2)
            
            bp1 = plt.boxplot(np.reshape(mag, -1))
            plt.show()
            
            mask1 = mag > bp1['whiskers'][1].get_data()[1][1]
            
            bp2 = plt.boxplot(mag[mask1])
            plt.show()
            
            mask = (mag > bp2['medians'][0].get_data()[1][0]).astype(np.uint8)
            
            kernel = np.ones((5,5),np.uint8)
            
            mask = cv2.morphologyEx(mask, cv2.MORPH_OPEN, kernel)
            mask = cv2.dilate(mask, kernel)
            
            
            
            
        if displayVideo == True:
            
            if count > frame_wait and calc_flow == True:

                if normalize_scale and normalize_frame_advance:                
                    frame_to_diplay = fns.draw_flow(frame1_gray,
                                                    flow * (1 / scaleFactor) \
                                                    / framesToAdvance,
                                                    mask, step=4)
                elif normalize_scale:                
                    frame_to_diplay = fns.draw_flow(frame1_gray,
                                                    flow * (1 / scaleFactor),
                                                    mask, step=4)
                elif normalize_frame_advance:                
                    frame_to_diplay = fns.draw_flow(frame1_gray,
                                                    flow / framesToAdvance,
                                                    mask, step=4)
                else:                
                    frame_to_diplay = fns.draw_flow(frame1_gray, flow,
                                                    mask, step=4)  
            elif calc_flow == False:
                
                frame_to_display = frame1

            if insert_frame_number == True:
                
                number = overlay.form_number(count - framesToAdvance, digits,
                                             frame_to_display.shape[0:2])
                
                connectivity = 4
                cc_placed = cv2.connectedComponentsWithStats(number,
                                                             connectivity,
                                                             cv2.CV_32S)
                                
                frame_to_display = overlay.overlay(frame_to_display, cc_placed,
                                                  np.unique(cc_placed[1])[1:],
                                                  (0, 255, 0), alpha=1)
                            
            cv2.imshow(title, frame_to_display)

        saved_frames.append(frame_to_display)
        
        if len(saved_frames) > number_saved:
            saved_frames.pop(0)
        else:
            ind = ind + 1


        # update frame1
        frame1 = frame2
        frame1_gray = frame2_gray
                
        # Esc key press will escape
        k = cv2.waitKey(1) & 0xff
        if k == 27:

            # release capture
            cap.release()

            # close all the OpenCV windows
            cv2.destroyAllWindows()
            
            break
        # space key will pause
        elif k == 32:
            
            k = -1
            
            # until we get another space key
            allow_scrolling = True
            while(allow_scrolling):
            
                k = cv2.waitKey(1)
                
                # space or escape
                if k == 32 or k == 27:

                    allow_scrolling = False
                
                # left arrow
                elif k == 37:
                    
                    if ind > 0:
                        
                        ind = ind - 1
                        cv2.imshow(title, saved_frames[ind])
                
                # right arrow
                elif k == 39:
                    
                    if ind < len(saved_frames) - 1:
                        
                        ind = ind + 1
                        cv2.imshow(title, saved_frames[ind])
                               
            
#            According to "A Byte of Python", "The else block is executed when
#            the while loop condition becomes False - this may even be the first
#            time that the condition is checked. If there is an else clause for
#            a while loop, it is always executed unless you break out of the
#            loop with a break statement."
            
            else:
                
                # escape
                if k == 27:

                    # release capture
                    cap.release()
        
                    # close all the OpenCV windows
                    cv2.destroyAllWindows()
                    
                    break
                
                # space
                if k == 32:
                    
                    get_back = True
                    
                    while(get_back):
                        
                        if ind + 1 == len(saved_frames):
                            
                            get_back = False
                            
                        else:
                            
                            ind = ind + 1
                            
                            cv2.imshow(title, saved_frames[ind])
                            
                            k = cv2.waitKey(1)
                            
                            if k == 27:
                                
                                get_back = False

                    else:
                        
                        # escape
                        if k == 27:
        
                            # release capture
                            cap.release()
                
                            # close all the OpenCV windows
                            cv2.destroyAllWindows()
                            
                            break                            
             
    else:

        # release capture
        cap.release()

        # close all the OpenCV windows
        cv2.destroyAllWindows()

if video_write == True:
    
    out.release()

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)
